#!/bin/bash

statuscode=true

if [ -e "config.sh" ]; then
	if [ "$1" = "--unsetall" ] ; then
		source bin/unsetall.sh
		rm -f ./config.sh
		echo "variables were unset successfully"
		echo "please reconfigure"
	else
		source ./config.sh
	fi
else
	isrootfound=`which root-config`
	if [ "$isrootfound" = "" ]; then
		echo "root not setup."
		echo "please install it first."
		echo "note that after a proper installation,"
		echo "the root-config script should be in your PATH"
		statuscode=false
	fi

	islhapdffound=`which lhapdf-config`
	if [ "$islhapdffound" = "" ]; then
		echo "lhapdf not setup."
		echo "please install it first."
		echo "note that after a proper installation,"
		echo "the lhapdf-config script should be in your PATH"
		statuscode=false
	fi

	isfastjetfound=`which fastjet-config`
	if [ "$isfastjetfound" = "" ]; then
		echo "fastjet not setup."
		echo "please install it first."
		echo "note that after a proper installation,"
		echo "the fastjet-config script should be in your PATH"
		statuscode=false
	fi

	echo '#!/bin/bash' >> ./config.sh
	echo "" >> ./config.sh
	echo 'if [ -z "$MOSESSYS" ]; then' >> ./config.sh
	echo '   export MOSESSYS=$PWD' >> ./config.sh
	echo 'fi' >> ./config.sh

	heppdt=false
	pythia8=false	
	for arg in $* ; do
		if [ "x$arg" = "x--help" ] ; then
			echo "+---------------------------------------------------------------+"
			echo "| Options:                                                      |"
			echo "|   --help           : show this help message                   |"
			echo "|   --option=value   : see below                                |"
			echo "|                                                               |"
			echo "| Options with values (syntax: --option=value):                 |"
			echo "|   --heppdtpath=/your/path/to/heppdt                           |"
			echo "|   --pythia8path=/your/path/to/pythia8                         |"
			echo "+---------------------------------------------------------------+"
		elif [ "x$arg" = "x--unsetall" ] ; then
			source bin/unsetall.sh
			rm -f config.sh
			return 0
		elif [ x`echo x${arg} | grep "="` != x ] ; then
			option=`echo ${arg} | awk -F = '{print $1}'`
			value=`echo ${arg} | awk -F = '{print $2}'`
			if [ "x${option}" = "x--heppdtpath" ] ; then
				if [ "$value" = "" ] ; then
					echo "invalid value: $value"
					echo "heppdt not setup, set it with:"
					echo "   source configure.sh --heppdtpath=/your/path/to/heppdt"
					statuscode=false
				else
					if [ -d "$value" ] ; then
						echo 'if [ -z "$HEPPDTSYS" ]; then' >> ./config.sh
						echo "   export HEPPDTSYS=${value}"  >> ./config.sh
						echo 'fi' >> ./config.sh
						heppdt=true
					else
						echo "directory $value does not exist"
						echo "heppdt not setup, set it with:"
						echo "   source configure.sh --heppdtpath=/your/path/to/heppdt"
						statuscode=false
					fi
				fi
			elif [ "x${option}" = "x--pythia8path" ] ; then
				if [ "$value" = "" ] ; then
					echo "invalid value: $value"
					echo "pythia8 not setup, set it with:"
					echo "   source configure.sh --pythia8path=/your/path/to/pythia8"
					statuscode=false
				else
					if [ -d "$value" ] ; then
						libname=libpythia8.so
						if [ -e $value/lib/$libname ] ; then
							echo 'if [ -z "$PYTHIA8SYS" ]; then' >> ./config.sh
							echo "   export PYTHIA8SYS=${value}"  >> ./config.sh
							echo 'fi' >> ./config.sh
							pythia8=true
						else
							echo "the library $libname does not exist in $value/lib"
							echo "pythia8 should be configured with the --enable-shared option"
							echo "pythia8 not setup, set it with:"
							echo "   source configure.sh --pythia8path=/your/path/to/pythia8"
							statuscode=false
						fi
					else
						echo "directory $value does not exist"
						echo "pythia8 not setup, set it with:"
						echo "   source configure.sh --pythia8path=/your/path/to/pythia8"
						statuscode=false
					fi
				fi
			else
				echo "${arg}: wrong option. Ignored." >&2
			fi
		else
			echo "unknown option"
			echo "try 'source configure.sh --help'"
			statuscode=false
		fi
	done
	if ! $heppdt ; then
		echo "heppdt not setup, set it with:"
		echo "   source configure.sh --heppdtpath=/your/path/to/heppdt"
		statuscode=false
	fi
	if ! $pythia8 ; then
		echo "pythia8 not setup, set it with:"
		echo "   source configure.sh --pythia8path=/your/path/to/pythia8"
		statuscode=false
	fi
	echo ""  >> ./config.sh

	# set the PDFSETS, TBL & DATA variables *** mandatory for runtime *** both for moses and pythia8
	echo 'export LHAPDFSETS=`lhapdf-config --pdfsets-path`/' >> ./config.sh
	echo 'export LHAPATH=`lhapdf-config --pdfsets-path`/' >> ./config.sh
	echo 'export HEPPDTTBL=$HEPPDTSYS/data/' >> ./config.sh
	echo 'export PYTHIA8DATA=$PYTHIA8SYS/xmldoc' >> ./config.sh
	echo 'export MOSESDATA=`bin/moses-config --datadir`' >> ./config.sh
	echo ""  >> ./config.sh
	
	# set some local variables for moses
	echo 'mosespath=`bin/moses-config --prefix`' >> ./config.sh
	echo 'moseslibpath=`bin/moses-config --libdir`' >> ./config.sh
	echo 'mosesrootdictpath=`bin/moses-config --rootdictdir`' >> ./config.sh
	echo 'mosesuserlibpath=`bin/moses-config --userlibdir`' >> ./config.sh
	echo ""  >> ./config.sh

	# update LD_LIBRARY_PATH
	echo 'isheppdtld=`echo $LD_LIBRARY_PATH | grep $HEPPDTSYS/lib`' >> ./config.sh
	echo 'if [ "$isheppdtld" = "" ] ; then' >> ./config.sh
	echo '   export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:$HEPPDTSYS/lib' >> ./config.sh
	echo 'fi' >> ./config.sh

	echo 'ispythia8ld=`echo $LD_LIBRARY_PATH | grep $PYTHIA8SYS/lib`' >> ./config.sh
	echo 'if [ "$ispythia8ld" = "" ] ; then' >> ./config.sh
	echo '   export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:$PYTHIA8SYS/lib' >> ./config.sh
	echo 'fi' >> ./config.sh

	echo 'ismosesld=`echo $LD_LIBRARY_PATH | grep $moseslibpath`' >> ./config.sh
	echo 'if [ "$ismosesld" = "" ] ; then' >> ./config.sh
	echo '   export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:$moseslibpath' >> ./config.sh
	echo 'fi' >> ./config.sh

	echo 'ismosesarcld=`echo $LD_LIBRARY_PATH | grep $moseslibpath/archive`' >> ./config.sh
	echo 'if [ "$ismosesarcld" = "" ] ; then' >> ./config.sh
	echo '   export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:$moseslibpath/archive' >> ./config.sh
	echo 'fi' >> ./config.sh
	echo "" >> ./config.sh

	echo 'ismosesarcld=`echo $LD_LIBRARY_PATH | grep $mosesrootdictpath`' >> ./config.sh
	echo 'if [ "$ismosesarcld" = "" ] ; then' >> ./config.sh
	echo '   export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:$mosesrootdictpath' >> ./config.sh
	echo 'fi' >> ./config.sh
	echo "" >> ./config.sh

	echo 'echo "run ./helper --help"'  >> ./config.sh

	if ! $statuscode ; then
		rm -f config.sh
		echo "configuration failed"
		echo "try 'source configure.sh --help'"
	else
		echo "configured successfuly"
		echo "next time you log in, run 'source ./config.sh'"
		source ./config.sh
	fi
fi
