/********************************************************************************************/
/* an example program to demonstrate (in short description) the building of a cross section */
/********************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;

using namespace Moses1;

int main(int argc, char* argv[]) {

		cout << "Holy Moses..." << endl;

		TStyle* plotstyle = (TStyle*)gROOT->GetStyle("Plain");
		plotstyle->cd();

		// declare the application
		TApplication theApp("App", &argc, argv);

		commonOperations commonOps; // set the class for commonOperations call
		// set styles
		
		commonOps.SetOKStyle();


		cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
		parameters prm; // create an instance of the parameters class
		physicalParameters physPrm; // create an instance of the physical parameters struct
		runParameters runPrm; // create an instance of the run parameters struct
		prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
		prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
		////////////////////////////////////////E-Center Of Mass /////////////////////////////////////
		//runPrm.sMax=  7e3*7e3;//		
		runPrm.sMax = 14e3*14e3;
		//runPrm.sMax=  14e3*14e3;
		//////////////////////////////////////////////////////////////////////////////////////////////
		const int runtime = (int)((4100. - 1900.)/100.);
		double totalSigmaArray [runtime];
		double g_mGluonStarArray [runtime];
			string dirName = "/KKGvsSMGstudy/";

	string treeFilePath = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_theoryTree.root");

	/*double mStar = physPrm.mStar;
	cout << "choose: m^*=";
	cin >> mStar;
	physPrm.mStar = mStar;*/

	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;

	PDTparticle pdt_prt;
	pdt_prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	cout << "\n\n" << "-------Set all involved particle properties into 2 vectors--------" << endl;
	bool print = false;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	//int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP};

	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	// fill all quarks and leptons properties in a single vector
	pdt_prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for :
	int idBosons[] = {PDTGLU,PDTZ,PDTWPLUS}; // glu,  Z0, W+
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	pdt_prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector


	cout << "\n\n" << "-------Set some more properties manually into the 2 vectors-------" << endl;
	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here
	
	///////////////////LHAPDF/////////////////LHAPDF//////////////LHAPDF///////////////////////
	lhapdf lha; // set the class for lhapdf call											///
	// initialize lha (lhapdf instance)														///
	//string pdfSet = "MRST2001lo.LHgrid";													///
	string pdfSet = "MSTW2008lo90cl.LHgrid";												///
	//string pdfSet = "cteq65.LHgrid";														///					
																							///
																							///
	//string pdfSet = "cteq5l.LHgrid";														///	
	int subset = 0; // it is usually set to '0'												///
	lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with full path and name//
    ///////////////////////////////////////////////////////////////////////////////////////////

	// create relevant classes instances
	//couplings cplng; // set the class for couplings call
	couplings masiveGcouplings(physPrm);
	masiveGcouplings.setFGBqQbarCouplings(physPrm);
	//masiveGcouplings.printFGBqQbarCouplings();

	partialWidth wdt; // set the class for partialWidth call
	helicityAmplitude hAmp; // set the class for helicityAmplitude call
	rapidity rap;  // set the class for rapidity call
	
	physPrm.g_mGluonStar = 1.; //cuppeling defult value
	
	newZresonance zPrime("chi"); ////////////////?????????????????????
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	
	//--------------------------------------------------------------------//
		double interval = 0.5;
		int mass_cuppeling_run = 0;
		physPrm.mGluonStar = 1900.; // Ge
		// out particles 
		customParticle outFermion1 = CUSTOMfermions[5]; // propagator will decay into a PDTTOP PDTTOP_bar pair
		customParticle outFermion2 = CUSTOMfermions[5];
		//double outFermion1Helicity1 = 0.5; //right handed
		//double outFermion1Helicity2 = 0.5; 
		// const (look up) runtime = (int)((4000. - 2000.)/100.);
  for(int g_mGluonStar_i=1; g_mGluonStar_i <= runtime ; g_mGluonStar_i++){
	
	
	
	if (mass_cuppeling_run == 1)
	{
	// cuppeling run
  	    totalSigmaArray[g_mGluonStar_i-1]=0;
		g_mGluonStarArray[g_mGluonStar_i-1]=0;
		g_mGluonStarArray[g_mGluonStar_i-1] =  g_mGluonStar_i*interval;
						//	g_mGluonStarArray[g_mGluonStar_i]=((double)g_mGluonStar_i)*0.1;
		physPrm.g_mGluonStar = g_mGluonStarArray[g_mGluonStar_i-1]+0.5;
	}else
	{
		physPrm.mGluonStar = physPrm.mGluonStar + 100.; // Ge		
		g_mGluonStarArray[g_mGluonStar_i-1]= physPrm.mGluonStar;

	}
		cout << "Holy Moses..." << g_mGluonStar_i <<" " << physPrm.mGluonStar<<" " << physPrm.g_mGluonStar<<" "<<runPrm.sMax<<endl;

///////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////
	// for the runs
	dcomplex fullAmp_SM (0,0);
	dcomplex fullAmp_Zprime (0,0);
	
	dcomplex fullAmp_GKKSM (0,0);
	
	dcomplex fullAmp_GKK (0,0);
	dcomplex fullAmp_FGBRight (0,0);
	dcomplex fullAmp_FGBLeft (0,0);
	dcomplex fullAmp_FGBAll (0,0);
	

    dcomplex unitC (1,0);
	int nMax_ysegments = 100; // for the integration on the rapidity
	int nKKmodes = 1; // only first kk mode.
	double differentialSquareAmp_SM = 0;
	double differentialSquareAmp_KK = 0;
	double differentialSquareAmp_FGBRight = 0;
	double differentialSquareAmp_FGBLeft = 0;
	double differentialSquareAmp_FGBAll = 0;
	//double differentialSquareAmp_GKKSM = 0;


	double squareAmp_SM = 0; //masss less gloun
	double squareAmp_KK = 0;
	double squareAmp_FGBRight = 0;
	double squareAmp_FGBLeft = 0;
	double squareAmp_FGBAll = 0;

//	double squareAmp_GKKSM = 0;

	double totalwidth_GKK[3][9]; //2 helcity states 8 lambda metrixces,6 quarks, ([2][0][6] = 0)
	double totalwidth_Z0 = 0;

	double qqbarRapidity_shat = 0; // this is the rapidity integral factor (single integral)
	
	// the integral over all decay angles
	// [1] theta: distributing like ~ (1+/-cos(theta))^2
	// [2] phi: distributing uniformly
	double angularIntegral = 2.*physPrm.pi * 8./3.;  // integral over (1+cos(teta))^2d(cos(theta)) dphi

	double mHat_min = floor(2.05*CUSTOMfermions[5].pdt_prt.mass);
	//mHat_min = 1000.;
	double mHat_max = 6000.;
	double dmHat = 1.;
	double mHat = mHat_min;
	double s_hat = mHat_min*mHat_min;
	Int_t nentries = (Int_t)((mHat_max-mHat_min)/dmHat);
	cout << "nentries=" << nentries << endl;


	// alocate arrays for the graph:
	double *Sqrt_s_hat = (double *)malloc(nentries*sizeof(double));
	Double_t *Sqare_amp_SM = (Double_t *)malloc(nentries*sizeof(Double_t));
	double *Sqare_amp_KK = (double *)malloc(nentries*sizeof(double));
	double *Sqare_amp_FGBRight = (double *)malloc(nentries*sizeof(double));
	double *Sqare_amp_FGBLeft = (double *)malloc(nentries*sizeof(double));


	TFile *treeFile = new TFile(treeFilePath.c_str(),"RECREATE");
	string treeName = "theoryTree";
	TTree *theoryTree = new TTree(treeName.c_str(),treeName.c_str());

	theoryTree->Branch("mHat",&mHat,"mHat/D");
	theoryTree->Branch("dsigmadmHat_SM",&squareAmp_SM,"squareAmp_SM/D");
	theoryTree->Branch("dsigmadmHat_KK",&squareAmp_KK,"squareAmp_KK/D");
	theoryTree->Branch("dsigmadmHat_FGBRight",&squareAmp_FGBRight,"squareAmp_FGBRight/D");
	theoryTree->Branch("dsigmadmHat_FGBLeft",&squareAmp_FGBLeft,"squareAmp_FGBLeft/D");
///////////////////////////Cuppeling Check//getFGBqQbarCouplingsbyIndex(physicalParameters &physPrm, unsigned int Lambda_I,int &qfermion, double &qhelicity, int &Qfermion, double &Qhelicity)
	cout << "----------------- u t cuppelings ------------------------------"<<endl;
	for (int il=1;il<9;il++){
	int q=2;
	int Q=6;
	double h=0.5;
	cout <<"il-" <<il<<" "<<abs(masiveGcouplings.getFGBqQbarCouplingsbyIndex(physPrm,il,q,h,Q,h))<<endl;
	q=PDTTOP;
	Q=PDTTOP;
	cout <<"il-" <<il<<" -qQ-"<<q<<Q<<" "<<abs(masiveGcouplings.getFGBqQbarCouplingsbyIndex(physPrm,il,q,h,Q,h))<<endl;
	cout <<"il-" <<il<<" -qQ-"<<q<<Q<<" "<<abs(masiveGcouplings.getFGBqQbarCouplingsbyIndex(physPrm,il,q,-0.5,Q,-0.5))<<endl;
	}
	cout << "----------------- u t cuppelings ------------------------------"<<endl;


//exit(0);
	//theoryTree->Branch("dsigmadmHat_KK",&squareAmp_GKKSM,"squareAmp_GKKSM/D");

	cout << "\n\nRun over s_hat ([" << mHat_min << "->" << mHat_max << "]GeV)^2:" << endl;

				//for (double hOut1=-0.5 ; hOut1<=0.5 ; hOut1++) { // loop over incoming helicity states (all quarks)
				//	for (double =-0.5 ; hOut2<=0.5 ; hOut2++) { // loop over outgoing helicity states (all quarks)


	double hOut1=-0.5 ;
	double hOut2=+0.5 ;
	int Group = 0;


	double tempS=-1.;     //  tempS=-1.  this ignors the kinematic condition.
	/////////////////////////////////////////////////////////////////////////////////
	// total width calculation s-channel in_f = out_f
	////////////////////////////////////////////////////////////////////////////////
	nKKmodes=1;
	cout << "here1" ;
			for (int KKmode=1 ; KKmode<=nKKmodes ; KKmode++) { // loop over all KK modes .one KK mode only for this model
				for ( Group=0 ; Group<3 ; Group++){
					for (int Lambda_I=1 ; Lambda_I<9 ; Lambda_I++){ //running over gell mann metrixecs // every lambda needs a tota width
					   ////////////////////////////////////////////////////////////
					   //////////////////////////starting to sum per Massive - G/////////////////////////////
					   totalwidth_GKK[Group][Lambda_I] = 0.;
					   //	cout << "Group ([" << Group << "Lambda_I" << Lambda_I << ":" << endl;
						switch  (Group){
						 		case 0:{
									hOut1=-0.5;
									hOut2=-0.5;
								//	cout << "here2" ;

//	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};

									for (int i=0 ; i<6; i++) { // running over all of out going q - qbar pairs = s-channel.

									//	double tempS=0;     //  tempS=0  this ignors the kinematic condition.
										totalwidth_GKK[Group][Lambda_I] += wdt.getpartialMassiveGWidth(Lambda_I,KKmode,tempS,
																		 PDTbosons[0], CUSTOMfermions[i],CUSTOMfermions[i],
																		 hOut1,hOut2,
																		 physPrm, masiveGcouplings);
									//cout <<i<< "@"<<CUSTOMfermions[i].pdt_prt.id << "@";
									}//for
									break;								 
								}//case								
								case 1:{
									hOut1=+0.5;
									hOut2=+0.5;
									for (int i=0 ; i<6; i++) { // running over all of out going q - qbar pairs = s-channel.
									//	double tempS=0;     //  tempS=0  this ignors the kinematic condition.
										totalwidth_GKK[Group][Lambda_I] += wdt.getpartialMassiveGWidth(Lambda_I,KKmode,tempS,
																		 PDTbosons[0], CUSTOMfermions[i],CUSTOMfermions[i],
																		 hOut1,hOut2,
																		 physPrm, masiveGcouplings);
										//cout << i<<"#"<<CUSTOMfermions[i].pdt_prt.id << "#";
										i++; // down type																		
									}//for
									break;
//	nt idFermions[] = {PDTDWN 0 ,PDTUP 1 ,PDTSTR 2,PDTCHM 3 ,PDTBOT 4,PDTTOP 5,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTA

								}//case
								case 2:{
							//	(((q == 1)||(q==3)||(q==5))&&(hIn > 0.)) {
							//Group = 2; //up type r ;
									hOut1=+0.5;
									hOut2=+0.5;
									//	double tempS=0;     //  tempS=0  this ignors the kinematic condition.
									for (int i=1 ; i<6; i++) { // running over all of out going q - qbar pairs = s-channel.
										totalwidth_GKK[Group][Lambda_I] += wdt.getpartialMassiveGWidth(Lambda_I,KKmode,tempS,
																		 PDTbosons[0], CUSTOMfermions[i],CUSTOMfermions[i],
																		 hOut1,hOut2,
																		 physPrm, masiveGcouplings);
 										//cout << i<<"%"<<CUSTOMfermions[i].pdt_prt.id << "%";								 
										i++; // up type
									}//for
									break;								 
								}//case
								
							}//switch
					       	cout << "Group - " << Group << " Lambda_I" << Lambda_I << ":" << "total width is " << totalwidth_GKK[Group][Lambda_I] << " GeV" << endl;

					}//for Lambda_I
			  }//for (double Group
		}//for KKmode
////////////----------------------- END  TOTAL WIDTH -----------------------------------------------------------
////////////----------------------------------------------------------------------------------------------------

	int n;
	s_hat = mHat_min*mHat_min;
	//cout << s_hat << endl;
	//exit(0);
	for (n=0 ; n<nentries ; n++)
	{ // loop over s_hat
		squareAmp_SM = 0; // before each new s_hat it needs to be initialized to '0'
		squareAmp_KK = 0;
		squareAmp_FGBRight = 0;
		squareAmp_FGBLeft = 0;
		squareAmp_FGBAll = 0;
		totalwidth_Z0 = 0; // before each new s_hat it needs to be initialized to '0'

		for (int q=0 ; q<(int)CUSTOMfermions.size() ; q++) 
		{ // run over all incoming quarks pairs include  ttbar
			//----------------------------------------------------------------------------------------//
			differentialSquareAmp_SM		= 0.; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_FGBRight  = 0.; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_FGBLeft 	= 0.; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_FGBAll 	= 0.; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_KK 		= 0.; // new FGB only
			//----------------------------------------------------------------------------------------//
			for (double hIn=-0.5,hOut=-0.5 ; hIn<=0.5 ; hIn++,hOut++) { // loop over incoming helicity states (all quarks)		
				// ---------- KK terms ----------------------------------------------------------------------
				// before each helicity state this has to be initialized to '(0,0)'
				fullAmp_GKK 	 = dcomplex(0,0); // Masssive G total width with adding the SM width  as well
				fullAmp_FGBRight = dcomplex(0,0);
				fullAmp_FGBLeft  = dcomplex(0,0);				
					// if the Helicity are the same, then the FGB can be produce.
					if (hIn == hOut)
					{					
						 if (hIn < 0.) {
							Group = 0; // left Group						    
						 }//if
						 else if  (((q == 0)||(q == 2)||(q == 4))&&(hIn > 0.))
						 { //{PDTDWN 0 ,PDTUP 1 ,PDTSTR 2,PDTCHM 3 ,PDTBOT 4,PDTTOP 5}
							Group = 1; //down type right ;							
						 }//if
						 else if  (((q == 1)||(q == 3)||(q == 5))&&(hIn > 0.)) 
						 {//{PDTDWN 0 ,PDTUP 1 ,PDTSTR 2,PDTCHM 3 ,PDTBOT 4,PDTTOP 5}
							Group = 2; //up type right ;							
						 }//if
						 
						 //   tt bar final state only... so no need to run on out going quarks
							 for (int Lambda_I =1 ;Lambda_I <9;Lambda_I++)
							 { //summing over all of the glouns in this group // sum on a index per q
								// only group 2 and 1 contribute to ttbar cross-section
								if (Group == 2)
								{// no interference between the group 
									fullAmp_FGBRight +=
									hAmp.gethelicityMassiveGAmplitude(Lambda_I, nKKmodes, s_hat, totalwidth_GKK[Group][Lambda_I],
									hIn, hIn, hOut, hOut, 
									CUSTOMfermions[q] , CUSTOMfermions[q], outFermion1, outFermion2, 
									PDTbosons[0], physPrm, masiveGcouplings);
								}
								
								if (Group == 0)
								{// no interference between the 
									fullAmp_FGBLeft+=
									hAmp.gethelicityMassiveGAmplitude(Lambda_I, nKKmodes, s_hat, totalwidth_GKK[Group][Lambda_I],
									hIn, hIn, hOut, hOut, 
									CUSTOMfermions[q] , CUSTOMfermions[q], outFermion1, outFermion2, 
									PDTbosons[0], physPrm, masiveGcouplings);
								}
							}//for
							if (fullAmp_GKK.real() > 1.)
							{// Error msg, someting is wrong
								cout << "  f "<<fullAmp_GKK<< " n " << n << " q "<< q << "differentialSquareAmp_KK"<<differentialSquareAmp_KK ;
							}
					    // get the differential square amplitude: |fullAmp|^2
						differentialSquareAmp_FGBRight    +=  real( fullAmp_FGBRight * conj(fullAmp_FGBRight) );
						differentialSquareAmp_FGBLeft     +=  real( fullAmp_FGBLeft  * conj(fullAmp_FGBLeft) );
						differentialSquareAmp_KK     	  +=  differentialSquareAmp_FGBRight + differentialSquareAmp_FGBLeft;	
						// -----------------------------------------------------------------------------------------//
					}//if Hin== Hout
					
					differentialSquareAmp_SM     += (s_hat*s_hat) * (1./4.) * real( fullAmp_SM*conj(fullAmp_SM) );					//} // end hOut
			} // end Hin Hout
			
			// the qqbar rapidity		integral factor
			qqbarRapidity_shat = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha); 
			
			if (qqbarRapidity_shat < 0.)
			{ // Error Msg
				cout <<" Rapidity integral < 0 and = " << qqbarRapidity_shat << endl;
			}
			// [1] multyply by the rapidity integral factor of the corresponding incoming partons (q+qbar)
			// [2] divide by the number of colors of q since the PDF functions return 9 color combinations
			//     and here only 3 are valid sice the propogator is colorless
			// [3] miltiply by 2 to account the 2 possible origins of the q or the qbar in the beams
			differentialSquareAmp_SM       *= 2.*qqbarRapidity_shat/(double)CUSTOMfermions[q].Nc;
			differentialSquareAmp_KK       *= 2.*qqbarRapidity_shat/(double)CUSTOMfermions[q].Nc;
			differentialSquareAmp_FGBRight *= 2.*qqbarRapidity_shat/(double)CUSTOMfermions[q].Nc;
			differentialSquareAmp_FGBLeft  *= 2.*qqbarRapidity_shat/(double)CUSTOMfermions[q].Nc;
			//---------------------------------------------------------------------------------------------//
			// only then add the differential square amplitude to the full one
			squareAmp_KK     	+= differentialSquareAmp_KK;
			squareAmp_FGBRight  += differentialSquareAmp_FGBRight;
			squareAmp_FGBLeft   += differentialSquareAmp_FGBLeft;


		} // end q //differentialSquareAmp_SM and differentialSquareAmp_KK are set to 0 in the beginning of this loop
	/*	if ((mHat >3999.)&&(mHat < 4001.)){
						cout <<"------------------------------------------------------------------------------"<<endl;
						cout <<"---------------"<<mHat<<endl;
						cout <<"-----------------------------------------------------------------------------"<<endl;						
							cout << "SquareAmp_FGBRight="<<squareAmp_FGBRight<<endl;
							cout <<"SquareAmp_FGBLeft="<<squareAmp_FGBLeft<<endl;
							cout << "SquareAmp_KK="<<squareAmp_KK<<endl;
						}		*/
		// multiply by the coefficients appearing before the squared helicity amplitude
		//
		//squareAmp_SM     *= 2.*sqrt(s_hat); // dSigma(s_hat) -> dSigma(sqrt(s_hat)) Jacobian of the transformation
															  //angularIntegral = 2.*physPrm.pi * 8./3.
		squareAmp_KK   		  *=s_hat*(1./(4*4*(4*4* physPrm.pi*physPrm.pi))) * angularIntegral * (double)outFermion1.Nc ; //256 = 4*4*4*4
		squareAmp_KK   		  *= 2.*sqrt(s_hat); // change of t to sqrt(s) = 2*sqrt(2) 

		squareAmp_FGBRight    *=(1./(256.* physPrm.pi*physPrm.pi)) * angularIntegral * (double)outFermion1.Nc ; //256 = 4*4*4*4
		squareAmp_FGBRight    *= 2.*sqrt(s_hat) * s_hat; // change of t to sqrt(s) = 2*sqrt(2) 

		squareAmp_FGBLeft     *=(1./(256.* physPrm.pi*physPrm.pi)) * angularIntegral * (double)outFermion1.Nc ; //256 = 4*4*4*4
		squareAmp_FGBLeft     *= 2.*sqrt(s_hat) * s_hat; // change of t to sqrt(s) = 2*sqrt(2) 
		// [squareAmp] = [1/GeV^3]
	//	cout <<"squareAmp_KK"<<squareAmp_KK<<"=--=-=lo=-";
		squareAmp_KK   		  *= physPrm.GeV2nbarn;
		squareAmp_FGBRight    *= physPrm.GeV2nbarn;
		squareAmp_FGBLeft     *= physPrm.GeV2nbarn;		
		//cout <<"squareAmp_KK"<<squareAmp_KK<<"=--=-=lo=-"; exit(0);
     	// [squareAmp] = [nb/GeV]
		// see more details about the conversion using the method "prm.printUnits()" in the end of the loop.

		// fill arrays of graphs:
	//	Sqare_amp_SM[n]     = squareAmp_SM;
		Sqare_amp_SM[n]     	= 0.;
		Sqare_amp_KK[n]     	= squareAmp_KK;
		Sqare_amp_FGBRight[n]   = squareAmp_FGBRight;
		Sqare_amp_FGBLeft[n]    = squareAmp_FGBLeft;
	//	Sqare_amp_GKKSM[n]  = squareAmp_GKKSM;
		//if(squareAmp_SM<=0 || squareAmp_KK<=0)
		if(squareAmp_KK<=0)
		{//Error msg, something is wrong.
			//cout << "n=" << n << ", squareAmp_KK=" << squareAmp_KK << ", squareAmp_SM=" << squareAmp_SM << endl;
		}

		Sqrt_s_hat[n] = mHat;
		theoryTree->Fill();
		mHat += dmHat;
		s_hat = mHat*mHat;
	} // end sMat
	prm.printUnits(); // just a reminder about units...
	theoryTree->Print();
	treeFile->Write();
	
	double totalSigma=0;
	cout <<endl << "dmhat" << dmHat<< endl;
	cout <<"1: "<< Sqare_amp_FGBRight[1] << endl;
	cout <<"2: "<< Sqare_amp_FGBRight[2] << endl;
	for (int ii=0; ii<nentries ; ii++)
	{	
        totalSigma+=Sqare_amp_FGBRight[ii]*dmHat;
	}
    //totalSigma = commonOps.simpsonIntegration(Sqare_amp_KK , Sqrt_s_hat, nentries ,1000.,5999.);
	//totalSigma = commonOps.simpsonIntegration(Sqare_amp_FGBRight , Sqrt_s_hat, nentries ,1000.,5999.);
	cout << " totalSigma by simple integral" << totalSigma << " physPrm.g_mGluonStar = " << physPrm.g_mGluonStar << endl;
	
	//totalSigma = commonOps.simpsonIntegration(Sqare_amp_FGBRight , Sqrt_s_hat, nentries ,1000.,5999.);
	cout << " totalSigma by simpson" << totalSigma << " physPrm.g_mGluonStar = " << endl;
	
	totalSigmaArray[g_mGluonStar_i-1]=totalSigma;
	
	 
	double tt=0;
	for (int ii=0; ii<nentries ; ii++)
	{	
		tt+=Sqare_amp_KK[ii]*dmHat;
	}
	cout <<" simple integral chack all = "<<tt<<endl;
	tt=0;
	
	//cout <<" simple integral chack right= "<<tt<<endl;
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	stringstream out;
	out << "R"<<g_mGluonStar_i<<"M"<<physPrm.mGluonStar<<"C"<<physPrm.g_mGluonStar;
	cout << "\n\n" << "--------------creating graph--------------" << endl;
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"avi"+out.str()+"_SigmavMass.root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"avi"+out.str()+"_SigmavMass.eps");
	TFile *rootFile = new TFile(rootFileName.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph

	TCanvas *c1 = new TCanvas("c1","qqbar-->FGB-->T Tbar  dSigma(s)/ds    ",0,0,1000,600);
	c1->SetLogy(1);

	TGraph *crossSection_KK = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_KK);
	TGraph *crossSection_FGBRight = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_FGBRight);
	TGraph *crossSection_FGBLeft = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_FGBLeft);

	//crossSection_KK->SetMinimum( 0.9*crossSection_KK->GetMinimum() );
	//crossSection_KK->SetMaximum( 1.1*crossSection_KK->GetMaximum() );
/*	double maxKK,minKK;
	maxKK = Sqare_amp_KK[0];
	minKK = Sqare_amp_KK[0];
	for (int l=1;l<nentries;l++){
		maxKK = (Sqare_amp_KK[l] > maxKK)? Sqare_amp_KK[l] : maxKK ;
		minKK = (Sqare_amp_KK[l] < minKK)? Sqare_amp_KK[l] : minKK ;
		
	}
	cout << "Min " << minKK << "Max " << maxKK <<endl;
    crossSection_KK->SetMinimum( 0.9*minKK);
	crossSection_KK->SetMaximum( 1.1*maxKK);
*/
//	crossSection_SM->SetTitle("dSigma(s)/ds[pp-->mu-mu+]   SM vs. E6 vs. KK");
	out.str("");
	//out.clear();
    //out <<"(FGB factor ="<<physPrm.g_mGluonStar <<",total cross-section in nb =" << totalSigma <<",FGB mass = "<<physPrm.mGluonStar<<")";

	out.clear();
    out <<"Total cross-section in nb = " << totalSigma ;

    string stitle;
	stitle = "dSigma(s)/ds[pp-->ttbar]  vs. FCGB " + out.str();
	crossSection_KK->SetTitle(stitle.c_str());
	//ossSection_KK->SetTitle("");
	crossSection_KK->GetXaxis()->SetTitle("#sqrt{#hat s} [GeV]");
	crossSection_KK->GetYaxis()->SetTitle("#frac{d#sigma}{d#sqrt{#hat s}} [#frac{nb}{GeV}]");
	crossSection_KK->Draw("AC");
	crossSection_KK->SetLineStyle(1);
	crossSection_KK->Draw("C");
		
	crossSection_FGBRight->SetLineStyle(3);
	crossSection_FGBRight->SetLineColor(kBlack);
	crossSection_FGBRight->Draw("C");

	crossSection_FGBLeft->SetLineStyle(1);
	crossSection_FGBLeft->SetLineColor(kRed);
	crossSection_FGBLeft->Draw("C");
	
	TLegend *leg = new TLegend(0.6, 0.7, 0.98, 0.9);
	leg->SetFillColor(5);
	leg->AddEntry(crossSection_FGBRight,"FGB(Right)","L");
	leg->AddEntry(crossSection_FGBLeft,"FGB(Left)","L");
	leg->AddEntry(crossSection_KK,"FGB-All","L");
	leg->Draw();

	//string text1;
    // ostringstream ss;
    // ss <<physPrm.g_mGluonStar <<",total cross-section in nb = " << totalSigma; ;
    //text1 = ss.str();
    //text1 = "residal LS a: "+text1;  
	//c1->SetLogy();
	c1->Print(rootFileName.c_str(),"root");
	c1->Update();
	
	c1->Print(epsFileName.c_str());
	rootFile->Write();

	cout << "\n" << "done..." << endl;

	// wait so the plots can hold on
      //  c1->WaitPrimitive();
	 

	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	stringstream out;
	out << "SigmaVSg" ;
	cout << "\n\n" << "--------------creating graph--------------" << endl;
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"avi"+out.str()+"_invMass_theory.root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"avi"+out.str()+"_invMass_theory.eps");
	TFile *rootFile = new TFile(rootFileName.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph
	
	//TCanvas *c2 = new TCanvas("c2","Inegrated cross Section Vs FCGB coupling",0,0,1000,600);
	TCanvas *c2 = new TCanvas("c2","Inegrated cross Section Vs FGB Mass",0,0,1000,600);
	//c1->SetLogy(1);
	c2->SetLogy(1);
	//c1->SetGrid();
	TGraph *crossSection_total= new TGraph (runtime, g_mGluonStarArray, totalSigmaArray);
    string stitle;
	stitle = "Inegrated cross Section Vs FGB Mass" ;
	//crossSection_total->SetTitle(stitle.c_str());
	crossSection_total->SetTitle("");
//	crossSection_total->SetLineColor(kRed);
	crossSection_total->SetLineStyle(1);
	crossSection_total->GetXaxis()->SetTitle("m_{FGB}");
	crossSection_total->GetYaxis()->SetTitle("#sigma [nb]");

	crossSection_total->Draw("AC");
//	TLegend *leg = new TLegend(0.6, 0.7, 0.98, 0.9);
//	leg->SetFillColor(5);
//	leg->AddEntry(crossSection_total,"Sigma","L");
//	leg->Draw();
	c2->Print(rootFileName.c_str(),"root");
	c2->Update();	
	c2->Print(epsFileName.c_str());
	rootFile->Write();
	cout << "\n" << "done..." << endl;
	// wait so the plots can hold on
      //  c1->WaitPrimitive();
	
	return 0;
}

/*
if ((q == 1)&&(Lambda_I==8))
									{
									
									cout << "------------------------------------------------------------------------------"<<endl;									
									//cout << endl<<abs(masiveGcouplings.getFGBqQbarCouplings(PDTbosons[0],physPrm,Lambda_I,CUSTOMfermions[q],hIn,CUSTOMfermions[q],hIn)*masiveGcouplings.getFGBqQbarCouplings(PDTbosons[0],physPrm,Lambda_I,outFermion1,hIn,outFermion2,hIn)) <<endl;
									cout << endl<<abs(masiveGcouplings.getFGBqQbarCouplings(PDTbosons[0],physPrm,Lambda_I,CUSTOMfermions[q],hIn,CUSTOMfermions[q],hIn));

									
									cout << "outFermion1 = " << outFermion1.pdt_prt.id << " outFermion2 = "<<outFermion2.pdt_prt.id << endl <<"CUSTOMfermions[q] " << CUSTOMfermions[q].pdt_prt.id << endl;
										cout << abs(hAmp.gethelicityMassiveGAmplitude(Lambda_I, nKKmodes, s_hat, totalwidth_GKK[Group][Lambda_I],
										hIn, hIn, hOut, hOut, 
										CUSTOMfermions[q] , CUSTOMfermions[q], outFermion1, outFermion2, 
										PDTbosons[0], physPrm, masiveGcouplings)) << endl;
										exit(0);
									}

									
									*/

