/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------
//#include "diLeptonBasicAnalyzer.h"

using namespace std;
using namespace Moses1;


int main() {
	cout << "Holy Moses..." << endl;
	
	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();

	gStyle->SetOptStat(11);
	gStyle->SetErrorX(0.5);

	// set cuts
	double invMassCut=1000.;
	double pTransverseCut=20.;
	double EtaCut=2.5;

	// declare some histos and related root entities
	commonOperations commonOps;
	string outFileName = "/ISRstudy/KK_ISR";
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".eps");

	TFile *histFile = new TFile(rootFileName.c_str(),"recreate");
	TCanvas *c1 = new TCanvas("c1","Kaluza-Klein",0,0,1400,800);
	c1->Divide(1,2);

	// plots of analysis on muon pairs
	TVirtualPad *pad1=c1->cd(1);
	pad1->SetLogy();
	//pad1->SetLogx(1);

	TVirtualPad *pad2=c1->cd(2);
	pad2->SetLogy();
	//pad2->SetLogx(1);

	// linear boundries and bins of histograms
	Int_t    invMnBins = 40;
	Double_t invMmin   = (Double_t)invMassCut;
	Double_t invMmax   = 6000.;

	Int_t    pTnBins = 40;
	Double_t pTmin   = (Double_t)pTransverseCut;
	Double_t pTmax   = 2000.;

	//----------------------------------------------------------------------------------
	// logarithmic boundries and bins of histograms
	Double_t logMmin = log10(invMmin);
	Double_t logMmax = log10(invMmax);
	Double_t M_binwidth = (Double_t)( (logMmax-logMmin)/(Double_t)invMnBins );
	Double_t M_bins[invMnBins+1];
	M_bins[0] = invMmin;
	for (Int_t i=1 ; i<=invMnBins ; i++) { M_bins[i] = TMath::Power( 10,(logMmin + i*M_binwidth) ); }

	Double_t logpTmin = log10(pTmin);
	Double_t logpTmax = log10(pTmax);
	Double_t pT_binwidth = (Double_t)( (logpTmax-logpTmin)/(Double_t)pTnBins );
	Double_t pT_bins[pTnBins+1];
	pT_bins[0] = pTmin;
	for (Int_t i=1 ; i<=pTnBins ; i++) { pT_bins[i] = TMath::Power( 10,(logpTmin + i*pT_binwidth) ); }
	//----------------------------------------------------------------------------------


	// declaration of histograms
	//TH1 *invMass_withISR = new TH1D("m(mu-,mu+) with ISR", "muon pairs invariant mass with ISR", invMnBins, invMmin, invMmax);
	TH1 *invMass_withISR = new TH1D("m(mu-,mu+) with ISR", "muon pairs invariant mass with ISR", invMnBins, M_bins);
	invMass_withISR->SetLineColor(kBlack);
	invMass_withISR->SetLineWidth(1);
	invMass_withISR->SetXTitle("invariantMass(mu-,mu+)[GeV]");
	invMass_withISR->SetYTitle("Events/bin");
	invMass_withISR->SetTitle("Muon pairs invariant mass distribution with/without ISR");
	invMass_withISR->SetMarkerStyle(20);
	invMass_withISR->SetMarkerColor(kBlack);

	//TH1 *invMass_noISR = new TH1D("m(mu-,mu+) no ISR", "muon pairs invariant mass no ISR", invMnBins, invMmin, invMmax);
	TH1 *invMass_noISR = new TH1D("m(mu-,mu+) no ISR", "muon pairs invariant mass no ISR", invMnBins, M_bins);
	invMass_noISR->SetLineColor(kRed);
	invMass_noISR->SetLineWidth(1);
	invMass_noISR->SetMarkerStyle(20);
	invMass_noISR->SetMarkerColor(kRed);

	// legends
	TLegend *leg1 = new TLegend(0.83,0.6,0.98,0.75);
	leg1->AddEntry(invMass_withISR,"with ISR", "L");
	leg1->AddEntry(invMass_noISR,"without ISR","L");
	leg1->SetFillColor(5);

	// do the same for pT
	//TH1 *pT_withISR = new TH1D("pT(mu-,mu+) with ISR", "muon pairs pT with ISR", pTnBins, pTmin, pTmax);
	TH1 *pT_withISR = new TH1D("pT(mu-,mu+) with ISR", "muon pairs pT with ISR", pTnBins, pT_bins);
	pT_withISR->SetLineColor(kBlack);
	pT_withISR->SetLineWidth(1);
	pT_withISR->SetXTitle("pT(mu-,mu+)[GeV]");
	pT_withISR->SetYTitle("Events/bin");
	pT_withISR->SetTitle("Muon pairs pT distribution with/without ISR");
	pT_withISR->SetMarkerStyle(20);
	pT_withISR->SetMarkerColor(kBlack);

	//TH1 *pT_noISR = new TH1D("pT(mu-,mu+) no ISR", "muon pairs pT no ISR", pTnBins, pTmin, pTmax);
	TH1 *pT_noISR = new TH1D("pT(mu-,mu+) no ISR", "muon pairs pT no ISR", pTnBins, pT_bins);
	pT_noISR->SetLineColor(kRed);
	pT_noISR->SetLineWidth(1);
	pT_noISR->SetMarkerStyle(20);
	pT_noISR->SetMarkerColor(kRed);

	TLegend *leg2 = new TLegend(0.83,0.6,0.98,0.75);
	leg2->AddEntry(invMass_withISR,"with ISR", "L");
	leg2->AddEntry(invMass_noISR,"without ISR","L");
	leg2->SetFillColor(5);

	//----------------------------------------------------------------------------------------------------------
	// declare the relevant classes
	dileptonAnalyzer anlz;
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);
	
	// start loop over events file(/files)
	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons, Naleptons;
	double invariantMass, pTransverse;

	// loop over 2 files (with / without Initial State Radiation)
	for (int f=1 ; f<=2 ; f++) {
		if (f==1) { filePath = commonOps.checkANDsetFilepath("MOSESDATA", "/ISRstudy/KK_extractedParticles_withISR.dat"); }
		else if (f==2) { filePath = commonOps.checkANDsetFilepath("MOSESDATA", "/ISRstudy/KK_extractedParticles_noISR.dat"); }
		Nevents = anlz.loopOverEvents(filePath);
		cout << "Nevents = " << Nevents << endl;

		// get all the good pairs (after cuts)
		Nleptons = anlz.getAllLeptons(allgoodLeptons);
		Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
		cout << "Nleptons = " << Nleptons << endl;
		if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
	
		// run over the good pairs and fill the histograms
		for (int k=0 ; k<Nleptons ; k++) {
			invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
			pTransverse = anlz.pT(allgoodLeptons[k], allgoodaLeptons[k]);
			if (f==1) {
				invMass_withISR->Fill( (Double_t)invariantMass );
				pT_withISR->Fill( (Double_t)pTransverse );
			}
			else if (f==2) {
				invMass_noISR->Fill( (Double_t)invariantMass );
				pT_noISR->Fill( (Double_t)pTransverse );
			}
		}
	
		// clear the pairs vectors
		anlz.clearAllLeptons();
		if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
		if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
	}
	//----------------------------------------------------------------------------------------------------------

	// divide by the bin width
	Double_t binSize, binValue, binError;
	for (Int_t b=1 ; b<=invMnBins ; b++) {
		binValue = invMass_withISR->GetBinContent(b);
		binError = invMass_withISR->GetBinError(b);
		binSize = invMass_withISR->GetBinLowEdge(b+1) - invMass_withISR->GetBinLowEdge(b);
		invMass_withISR->SetBinContent(b,binValue/binSize);
		invMass_withISR->SetBinError(b,binError/binSize);

                binValue = invMass_noISR->GetBinContent(b);
		binError = invMass_noISR->GetBinError(b);
                binSize = invMass_noISR->GetBinLowEdge(b+1) - invMass_noISR->GetBinLowEdge(b);
                invMass_noISR->SetBinContent(b,binValue/binSize);
		invMass_noISR->SetBinError(b,binError/binSize);
	}
	for (Int_t b=1 ; b<=pTnBins ; b++) {
		binValue = pT_withISR->GetBinContent(b);
		binError = pT_withISR->GetBinError(b);
		binSize = pT_withISR->GetBinLowEdge(b+1) - pT_withISR->GetBinLowEdge(b);
		pT_withISR->SetBinContent(b,binValue/binSize);
		pT_withISR->SetBinError(b,binError/binSize);

		binValue = pT_noISR->GetBinContent(b);
		binError = pT_noISR->GetBinError(b);
		binSize = pT_noISR->GetBinLowEdge(b+1) - pT_noISR->GetBinLowEdge(b);
		pT_noISR->SetBinContent(b,binValue/binSize);
		pT_noISR->SetBinError(b,binError/binSize);
	}

	// reset the number of entries
	invMass_withISR->SetEntries( invMass_withISR->GetEntries() - invMnBins );
	invMass_noISR->SetEntries( invMass_noISR->GetEntries() - invMnBins );
	pT_withISR->SetEntries( pT_withISR->GetEntries() - pTnBins );
	pT_noISR->SetEntries( pT_noISR->GetEntries() - pTnBins );

	// draw the histos
	c1->cd(1);
	c1->Update();
	//invMass_withISR->GetXaxis()->SetMoreLogLabels();
	invMass_withISR->Draw();
	invMass_withISR->DrawCopy("e1sames");
	invMass_noISR->Draw("sames");
	invMass_noISR->DrawCopy("e1sames");
	leg1->Draw("sames");
	gPad->RedrawAxis();
	// fix the PaveStats location (the first one should be lowered down, so the 1st one will not sade it)
	c1->Update();
	TPaveStats *s1 = (TPaveStats*)invMass_withISR->FindObject("stats");
	s1->SetY1NDC(0.818); //new y start position
	s1->SetY2NDC(0.9); //new y end position
	
	c1->cd(2);
	c1->Update();
	//pT_withISR->SetMaximum(1e3);
	pT_withISR->Draw();
	pT_withISR->DrawCopy("e1sames");
	pT_noISR->Draw("sames");
	pT_noISR->DrawCopy("e1sames");
	leg2->Draw("same");
	gPad->RedrawAxis();
	// fix the PaveStats location (the first one should be lowered down, so the 1st one will not sade it)
	c1->Update();
	TPaveStats *s2 = (TPaveStats*)pT_withISR->FindObject("stats");
	s2->SetY1NDC(0.818); //new y start position
	s2->SetY2NDC(0.9); //new y end position

	// finalize the graphics
	c1->Print(rootFileName.c_str(),"root");
	c1->Update();
	c1->Print(epsFileName.c_str());
	histFile->Write();

	// Done.
	return 0;
}

