/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Moses1;
using namespace Pythia8;


int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
	string dirName = "/";
	string outFileName = dirName + "statTests_multiSamples";
	cout << "\n" << endl;


	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	bool doTrue = true;
	anlz.setTrue(doTrue);
	Particle particle;

	double minM = 200;
	double maxM = 4000;
	cout << "choose mHat min...";
	cin >> minM;
	cout << "choose mHat max...";
	cin >> maxM;
	
	runPrm.invMassCut = minM;

	// set cuts
	double invMassCut=minM;
	double pTransverseCut=10.;
	double EtaCut=2.5;
	int cutSelector = 1;
	cout << "select cuts to apply: no cuts (-1) all cuts (0), only mHat cut (1), only pT cut (2), only eta cut (3)..." << endl;
	cin >> cutSelector;
	switch(cutSelector) {
		case -1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 invMassCut = 0.;
			 break;
		case  0: break;
		case  1: pTransverseCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  2: invMassCut = 0.;
			 EtaCut = 1e10;
			 break;
		case  3: invMassCut = 0.;
			 pTransverseCut = 0.;
			 break;
		default: cout << "wrong option cutSelector = " << cutSelector << "\t set to default cutSelector=0" << endl;
			 break;
	}
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);

	
	int mStarSelector = 4;
	string sMStarSelector = "_mStar=4TeV";
	cout << "select m^* value: 3TeV(3), 4TeV(4)...";
	cin >> mStarSelector;
	if (mStarSelector==3) {
		sMStarSelector = "_mStar=3TeV";
		physPrm.mStar = 3000.;
		physPrm.mZprime = 3000.;
		physPrm.wZprime = 80.;
	}
	else {
		sMStarSelector = "_mStar=4TeV";
		physPrm.mStar = 4000.;
		physPrm.mZprime = 4000.;
		physPrm.wZprime = 120.;
	}
	outFileName += sMStarSelector;
	
	
	int nRefSelector = 1;
	string sRefName  = "SM";
	cout << "select large reference sample: SM(1), KK(2)...";
	cin >> nRefSelector;
	if (nRefSelector==1)
	{
		sRefName = "SM";
	}
	else if (nRefSelector==2)
	{
		sRefName = "KK";
	}
	outFileName += "_" + sRefName + "ref";
	
	

	string statTestsRootFileName         = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + ".root");
	string epsFileName_mHatKprobBinned   = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_mHatKprobBinned.eps");
	string epsFileName_mHatKprobUnbinned = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_mHatKprobUnbinned.eps");
	string epsFileName_mHatChi2          = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_mHatChi2.eps");
	string epsFileName_pTKprobBinned     = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_pTKprobBinned.eps");
	string epsFileName_pTKprobUnbinned   = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_pTKprobUnbinned.eps");
	string epsFileName_pTChi2            = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_pTChi2.eps");

	TFile *statTestsHistFile = new TFile(statTestsRootFileName.c_str(),"recreate");	
	
	TDirectory* dir_mHat = statTestsHistFile->mkdir("mHat");
	dir_mHat->cd();

	TDirectory* dir_mHat_KprobBinned = dir_mHat->mkdir("mHat_KprobBinned");
	dir_mHat_KprobBinned->cd();
	TCanvas *canv_mHat_KprobBinned = new TCanvas("canv_mHat_KprobBinned" ,"canv_mHat_KprobBinned",0,0,1400,800);
	
	TDirectory* dir_mHat_KprobUnbinned = dir_mHat->mkdir("mHat_KprobUnbinned");
	dir_mHat_KprobUnbinned->cd();
	TCanvas *canv_mHat_KprobUnbinned = new TCanvas("canv_mHat_KprobUnbinned" ,"canv_mHat_KprobUnbinned",0,0,1400,800);
	
	TDirectory* dir_mHat_Chi2 = dir_mHat->mkdir("mHat_Chi2");
	dir_mHat_Chi2->cd();
	TCanvas *canv_mHat_Chi2 = new TCanvas("canv_mHat_Chi2" ,"canv_mHat_Chi2",0,0,1400,800);
	

	TDirectory* dir_pT = statTestsHistFile->mkdir("pT");
	dir_pT->cd();
	
	TDirectory* dir_pT_KprobBinned = dir_pT->mkdir("pT_KprobBinned");
	dir_pT_KprobBinned->cd();
	TCanvas *canv_pT_KprobBinned = new TCanvas("canv_pT_KprobBinned" ,"canv_pT_KprobBinned",0,0,1400,800);
	
	TDirectory* dir_pT_KprobUnbinned = dir_pT->mkdir("pT_KprobUnbinned");
	dir_pT_KprobUnbinned->cd();
	TCanvas *canv_pT_KprobUnbinned = new TCanvas("canv_pT_KprobUnbinned" ,"canv_pT_KprobUnbinned",0,0,1400,800);
	
	TDirectory* dir_pT_Chi2 = dir_pT->mkdir("pT_Chi2");
	dir_pT_Chi2->cd();
	TCanvas *canv_pT_Chi2 = new TCanvas("canv_pT_Chi2" ,"canv_pT_Chi2",0,0,1400,800);

	// boundries and bins of histograms
	Int_t invMnBins = 19;
	Double_t invMmin = invMassCut, invMmax = maxM;
	Double_t invMassBoundToReduce = 0.;

	Int_t    pTnBins = 14; // 100
	Double_t pTmin   = 10.;
	Double_t pTmax   = 1200.;
	Double_t pTBoundToReduce = 0.;
	



	if (cutSelector==-1 || cutSelector==1 || cutSelector==2 || cutSelector==3) {
		pTnBins = 100;
		pTmin = 5.;
		pTmax = 100.;
		
		invMnBins = 500;
		invMmin = 5.;
		invMmax = 6000.;
	}


	// logarithmic boundries and bins of histograms
	Double_t logMmin = log10(invMmin);
	Double_t logMmax = log10(invMmax - invMassBoundToReduce);
	Double_t M_binwidth = (Double_t)( (logMmax-logMmin)/(Double_t)invMnBins );
	Double_t M_bins[invMnBins+1];
	M_bins[0] = invMmin;
	for (Int_t i=1 ; i<=invMnBins ; i++) { M_bins[i] = TMath::Power( 10,(logMmin + i*M_binwidth) ); }

	Double_t logpTmin = log10(pTmin);
	Double_t logpTmax = log10(pTmax - pTBoundToReduce);
	Double_t pT_binwidth = (Double_t)( (logpTmax-logpTmin)/(Double_t)pTnBins );
	Double_t pT_bins[pTnBins+1];
	pT_bins[0] = pTmin;
	for (Int_t i=1 ; i<=pTnBins ; i++) { pT_bins[i] = TMath::Power( 10,(logpTmin + i*pT_binwidth) ); }



	string sTitle = "";

	
	//----- SM ref
	dir_mHat->cd();
	sTitle = sRefName + "ref_invMass";
	TH1 *REF_invMass = new TH1D(sTitle.c_str(), "", invMnBins, invMmin, invMmax);
	sTitle = sRefName + "ref_pT";
	TH1 *REF_pT = new TH1D(sTitle.c_str(), "", pTnBins, pTmin, pTmax);
	
	REF_invMass->SetLineStyle(1);
	REF_invMass->SetLineColor(kRed);
	REF_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	REF_invMass->SetYTitle("Events");

	dir_pT->cd();
	REF_pT->SetLineStyle(1);
	REF_pT->SetLineColor(kRed);
	REF_pT->SetXTitle("p_{T}(#mu^{-})[GeV]");
	REF_pT->SetYTitle("Events");
	

	//----- SM -----------------------------------------------------------------------------------------------------
	dir_mHat->cd();
	TH1 *SM_invMass = new TH1D("SM_invMass", "", invMnBins, invMmin, invMmax);
	SM_invMass->SetLineStyle(5);
	SM_invMass->SetLineColor(kBlue);
	SM_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	SM_invMass->SetYTitle("Events");

	dir_pT->cd();
	TH1 *SM_pT = new TH1D("SM_pT", "", pTnBins, pTmin, pTmax);
	SM_pT->SetLineStyle(5);
	SM_pT->SetLineColor(kBlue);
	SM_pT->SetXTitle("p_{T}(#mu^{-})[GeV]");
	SM_pT->SetYTitle("Events");
	
	//----- KK -----------------------------------------------------------------------------------------------------
	dir_mHat->cd();
	TH1 *KK_invMass = new TH1D("KK_invMass", "", invMnBins, invMmin, invMmax);
	KK_invMass->SetLineStyle(1);
	KK_invMass->SetLineColor(kRed);
	KK_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	KK_invMass->SetYTitle("Events");

	dir_pT->cd();
	TH1 *KK_pT = new TH1D("KK_pT", "", pTnBins, pTmin, pTmax);
	KK_pT->SetLineStyle(1);
	KK_pT->SetLineColor(kRed);
	KK_pT->SetXTitle("p_{T}(#mu^{-})[GeV]");
	KK_pT->SetYTitle("Events");

	//----- Z' -----------------------------------------------------------------------------------------------------
	dir_mHat->cd();
	TH1 *Zprime_invMass = new TH1D("Zprime_invMass", "", invMnBins, invMmin, invMmax);
	Zprime_invMass->SetLineStyle(3);
	Zprime_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	Zprime_invMass->SetYTitle("Events");

	dir_pT->cd();
	TH1 *Zprime_pT = new TH1D("Zprime_pT", "", pTnBins, pTmin, pTmax);
	Zprime_pT->SetLineStyle(3);
	Zprime_pT->SetXTitle("p_{T}(#mu^{-})[GeV]");
	Zprime_pT->SetYTitle("Events");

	// ------ legends -----------------------------------------------------------------------------------------------------
	TLegend *leg_mHat = new TLegend(0.7,0.5,0.96,0.7);
	leg_mHat->AddEntry(SM_invMass,"SM","L");
	leg_mHat->AddEntry(KK_invMass,"KK","L");
	leg_mHat->AddEntry(Zprime_invMass,"Z'_{SM}","L");
	
	TLegend *leg_pT = new TLegend(0.7,0.7,0.96,0.9);
	leg_pT->AddEntry(SM_pT,"SM","L");
	leg_pT->AddEntry(KK_pT,"KK","L");
	leg_pT->AddEntry(Zprime_pT,"Z'_{SM}","L");


	//------- work histos -------------------------------------------------------------------------------------------------
	// statTests invariant mass histos
	dir_mHat->cd();
	sTitle = sRefName + "ref_mHat_statTests";
	TH1D *REF_mHat_statTests    = new TH1D(sTitle.c_str(), "", invMnBins, invMmin, invMmax);
	sTitle = sRefName + "ref_pT_statTests";
	TH1D *REF_pT_statTests    = new TH1D(sTitle.c_str(), "", pTnBins, pTmin, pTmax);
	
	TH1D *SM_mHat_statTests     = new TH1D("SM_mHat_statTests",     "", invMnBins, invMmin, invMmax);
	TH1D *KK_mHat_statTests     = new TH1D("KK_mHat_statTests",     "", invMnBins, invMmin, invMmax);
	TH1D *Zprime_mHat_statTests = new TH1D("Zprime_mHat_statTests", "", invMnBins, invMmin, invMmax);

	// statTests pT histos 
	dir_pT->cd();
	TH1D *SM_pT_statTests     = new TH1D("SM_pT_statTests",     "", pTnBins, pTmin, pTmax);
	TH1D *KK_pT_statTests     = new TH1D("KK_pT_statTests",     "", pTnBins, pTmin, pTmax);
	TH1D *Zprime_pT_statTests = new TH1D("Zprime_pT_statTests", "", pTnBins, pTmin, pTmax);
	
	
	
	//------- results of pseudo data histograms -------------------------------------------------------------------------------------------------
	Int_t    statTests_nBins = 20;
	Double_t statTests_min   = 0.;
	Double_t statTests_max   = 1.;
	
	//------ SM
	dir_mHat_KprobBinned->cd();
	TH1D* hSM_KprobBinned_mHat     = new TH1D("SM_KprobBinned","",statTests_nBins, statTests_min, statTests_max);
	hSM_KprobBinned_mHat->SetLineStyle(5);
	hSM_KprobBinned_mHat->SetLineColor(kBlue);
	hSM_KprobBinned_mHat->SetXTitle("Binned Kolmogorov Probability");
	hSM_KprobBinned_mHat->SetYTitle("Pseudo experiments");

	dir_mHat_KprobUnbinned->cd();
	TH1D* hSM_KprobUnbinned_mHat     = new TH1D("SM_KprobUnbinned_mHat","",statTests_nBins, statTests_min, statTests_max);
	hSM_KprobUnbinned_mHat->SetLineStyle(5);
	hSM_KprobUnbinned_mHat->SetLineColor(kBlue);
	hSM_KprobUnbinned_mHat->SetXTitle("Unbinned Kolmogorov Probability");
	hSM_KprobUnbinned_mHat->SetYTitle("Pseudo experiments");
	
	dir_mHat_Chi2->cd();
	TH1D* hSM_Chi2_mHat     = new TH1D("SM_Chi2_mHat","",statTests_nBins, statTests_min, statTests_max);
	hSM_Chi2_mHat->SetLineStyle(5);
	hSM_Chi2_mHat->SetLineColor(kBlue);
	hSM_Chi2_mHat->SetXTitle("#chi^{2}_{p-value}");
	hSM_Chi2_mHat->SetYTitle("Pseudo experiments");
	
	dir_pT_KprobBinned->cd();
	TH1D* hSM_KprobBinned_pT     = new TH1D("SM_KprobBinned_pT","",statTests_nBins, statTests_min, statTests_max);
	hSM_KprobBinned_pT->SetLineStyle(5);
	hSM_KprobBinned_pT->SetLineColor(kBlue);
	hSM_KprobBinned_pT->SetXTitle("Binned Kolmogorov Probability");
	hSM_KprobBinned_pT->SetYTitle("Pseudo experiments");
	
	dir_pT_KprobUnbinned->cd();
    TH1D* hSM_KprobUnbinned_pT     = new TH1D("SM_KprobUnbinned_pT","",statTests_nBins, statTests_min, statTests_max);
	hSM_KprobUnbinned_pT->SetLineStyle(5);
	hSM_KprobUnbinned_pT->SetLineColor(kBlue);
	hSM_KprobUnbinned_pT->SetXTitle("Unbinned Kolmogorov Probability");
	hSM_KprobUnbinned_pT->SetYTitle("Pseudo experiments");
	
	dir_pT_Chi2->cd();
    TH1D* hSM_Chi2_pT     = new TH1D("SM_Chi2_pT","",statTests_nBins, statTests_min, statTests_max);
	hSM_Chi2_pT->SetLineStyle(5);
	hSM_Chi2_pT->SetLineColor(kBlue);
	hSM_Chi2_pT->SetXTitle("#chi^{2}_{p-value}");
	hSM_Chi2_pT->SetYTitle("Pseudo experiments");


	//------ KK
	dir_mHat_KprobBinned->cd();
	TH1D* hKK_KprobBinned_mHat     = new TH1D("KK_KprobBinned","",statTests_nBins, statTests_min, statTests_max);
	hKK_KprobBinned_mHat->SetLineStyle(1);
	hKK_KprobBinned_mHat->SetLineColor(kRed);
	hKK_KprobBinned_mHat->SetXTitle("Binned Kolmogorov Probability");
	hKK_KprobBinned_mHat->SetYTitle("Pseudo experiments");

	dir_mHat_KprobUnbinned->cd();
	TH1D* hKK_KprobUnbinned_mHat     = new TH1D("KK_KprobUnbinned_mHat","",statTests_nBins, statTests_min, statTests_max);
	hKK_KprobUnbinned_mHat->SetLineStyle(1);
	hKK_KprobUnbinned_mHat->SetLineColor(kRed);
	hKK_KprobUnbinned_mHat->SetXTitle("Unbinned Kolmogorov Probability");
	hKK_KprobUnbinned_mHat->SetYTitle("Pseudo experiments");
	
	dir_mHat_Chi2->cd();
	TH1D* hKK_Chi2_mHat     = new TH1D("KK_Chi2_mHat","",statTests_nBins, statTests_min, statTests_max);
	hKK_Chi2_mHat->SetLineStyle(1);
	hKK_Chi2_mHat->SetLineColor(kRed);
	hKK_Chi2_mHat->SetXTitle("#chi^{2}_{p-value}");
	hKK_Chi2_mHat->SetYTitle("Pseudo experiments");
	
	dir_pT_KprobBinned->cd();
	TH1D* hKK_KprobBinned_pT     = new TH1D("KK_KprobBinned_pT","",statTests_nBins, statTests_min, statTests_max);
	hKK_KprobBinned_pT->SetLineStyle(1);
	hKK_KprobBinned_pT->SetLineColor(kRed);
	hKK_KprobBinned_pT->SetXTitle("Binned Kolmogorov Probability");
	hKK_KprobBinned_pT->SetYTitle("Pseudo experiments");
	
	dir_pT_KprobUnbinned->cd();
    TH1D* hKK_KprobUnbinned_pT     = new TH1D("KK_KprobUnbinned_pT","",statTests_nBins, statTests_min, statTests_max);
	hKK_KprobUnbinned_pT->SetLineStyle(1);
	hKK_KprobUnbinned_pT->SetLineColor(kRed);
	hKK_KprobUnbinned_pT->SetXTitle("Unbinned Kolmogorov Probability");
	hKK_KprobUnbinned_pT->SetYTitle("Pseudo experiments");
	
	dir_pT_Chi2->cd();
    TH1D* hKK_Chi2_pT     = new TH1D("KK_Chi2_pT","",statTests_nBins, statTests_min, statTests_max);
	hKK_Chi2_pT->SetLineStyle(1);
	hKK_Chi2_pT->SetLineColor(kRed);
	hKK_Chi2_pT->SetXTitle("#chi^{2}_{p-value}");
	hKK_Chi2_pT->SetYTitle("Pseudo experiments");
	
	
	//----- Z'
	dir_mHat_KprobBinned->cd();
	TH1D* hZprime_KprobBinned_mHat = new TH1D("Zprime_KprobBinned_mHat","",statTests_nBins, statTests_min, statTests_max);
	hZprime_KprobBinned_mHat->SetLineStyle(3);
	hZprime_KprobBinned_mHat->SetLineColor(kBlack);
	hZprime_KprobBinned_mHat->SetXTitle("Binned Kolmogorov Probability");
	hZprime_KprobBinned_mHat->SetYTitle("Pseudo experiments");
	
	dir_mHat_KprobUnbinned->cd();
	TH1D* hZprime_KprobUnbinned_mHat = new TH1D("Zprime_KprobUnbinned_mHat","",statTests_nBins, statTests_min, statTests_max);
	hZprime_KprobUnbinned_mHat->SetLineStyle(3);
	hZprime_KprobUnbinned_mHat->SetLineColor(kBlack);
	hZprime_KprobUnbinned_mHat->SetXTitle("Unbinned Kolmogorov Probability");
	hZprime_KprobUnbinned_mHat->SetYTitle("Pseudo experiments");
	
	dir_mHat_Chi2->cd();
	TH1D* hZprime_Chi2_mHat = new TH1D("Zprime_Chi2_mHat","",statTests_nBins, statTests_min, statTests_max);
	hZprime_Chi2_mHat->SetLineStyle(3);
	hZprime_Chi2_mHat->SetLineColor(kBlack);
	hZprime_Chi2_mHat->SetXTitle("#chi^{2}_{p-value}");
	hZprime_Chi2_mHat->SetYTitle("Pseudo experiments");
	
	dir_pT_KprobBinned->cd();
	TH1D* hZprime_KprobBinned_pT = new TH1D("Zprime_KprobBinned_pT","",statTests_nBins, statTests_min, statTests_max);
	hZprime_KprobBinned_pT->SetLineStyle(3);
	hZprime_KprobBinned_pT->SetLineColor(kBlack);
	hZprime_KprobBinned_pT->SetXTitle("Binned Kolmogorov Probability");
	hZprime_KprobBinned_pT->SetYTitle("Pseudo experiments");
	
	dir_pT_KprobUnbinned->cd();
    TH1D* hZprime_KprobUnbinned_pT = new TH1D("Zprime_KprobUnbinned_pT","",statTests_nBins, statTests_min, statTests_max);
	hZprime_KprobUnbinned_pT->SetLineStyle(3);
	hZprime_KprobUnbinned_pT->SetLineColor(kBlack);
	hZprime_KprobUnbinned_pT->SetXTitle("Unbinned Kolmogorov Probability");
	hZprime_KprobUnbinned_pT->SetYTitle("Pseudo experiments");
	
	dir_pT_Chi2->cd();
    TH1D* hZprime_Chi2_pT = new TH1D("Zprime_Chi2_pT","",statTests_nBins, statTests_min, statTests_max);
	hZprime_Chi2_pT->SetLineStyle(3);
	hZprime_Chi2_pT->SetLineColor(kBlack);
	hZprime_Chi2_pT->SetXTitle("#chi^{2}_{p-value}");
	hZprime_Chi2_pT->SetYTitle("Pseudo experiments");
	
	
	//------- legends
	string sLegSM     = "SM vs. "      + sRefName;
	string sLegKK     = "KK vs. "      + sRefName;
	string sLegZprime = "Z'_{SM} vs. " + sRefName;
	
	TLegend *leg_mHat_KprobBinned = new TLegend(0.55,0.5,0.8,0.7);
	leg_mHat_KprobBinned->AddEntry(hSM_KprobBinned_mHat, sLegSM.c_str(), "L");
	leg_mHat_KprobBinned->AddEntry(hKK_KprobBinned_mHat, sLegKK.c_str(), "L");
	leg_mHat_KprobBinned->AddEntry(hZprime_KprobBinned_mHat, sLegZprime.c_str(), "L");
	
	TLegend *leg_mHat_KprobUnbinned = new TLegend(0.55,0.5,0.8,0.7);
	leg_mHat_KprobUnbinned->AddEntry(hSM_KprobUnbinned_mHat, sLegSM.c_str(), "L");
	leg_mHat_KprobUnbinned->AddEntry(hKK_KprobUnbinned_mHat, sLegKK.c_str(), "L");
	leg_mHat_KprobUnbinned->AddEntry(hZprime_KprobUnbinned_mHat, sLegZprime.c_str(), "L");
	
	TLegend *leg_mHat_Chi2 = new TLegend(0.55,0.5,0.8,0.7);
	leg_mHat_Chi2->AddEntry(hSM_Chi2_mHat, sLegSM.c_str(), "L");
	leg_mHat_Chi2->AddEntry(hKK_Chi2_mHat, sLegKK.c_str(), "L");
	leg_mHat_Chi2->AddEntry(hZprime_Chi2_mHat, sLegZprime.c_str(), "L");
	
	TLegend *leg_pT_KprobBinned = new TLegend(0.55,0.5,0.8,0.7);
	leg_pT_KprobBinned->AddEntry(hSM_KprobBinned_pT, sLegSM.c_str(), "L");
	leg_pT_KprobBinned->AddEntry(hKK_KprobBinned_pT, sLegKK.c_str(), "L");
	leg_pT_KprobBinned->AddEntry(hZprime_KprobBinned_pT, sLegZprime.c_str(), "L");
	
	TLegend *leg_pT_KprobUnbinned = new TLegend(0.55,0.5,0.8,0.7);
	leg_pT_KprobUnbinned->AddEntry(hSM_KprobUnbinned_pT, sLegSM.c_str(), "L");
	leg_pT_KprobUnbinned->AddEntry(hKK_KprobUnbinned_pT, sLegKK.c_str(), "L");
	leg_pT_KprobUnbinned->AddEntry(hZprime_KprobUnbinned_pT, sLegZprime.c_str(), "L");
	
	TLegend *leg_pT_Chi2 = new TLegend(0.55,0.5,0.8,0.7);
	leg_pT_Chi2->AddEntry(hSM_Chi2_pT, sLegSM.c_str(), "L");
	leg_pT_Chi2->AddEntry(hKK_Chi2_pT, sLegKK.c_str(), "L");
	leg_pT_Chi2->AddEntry(hZprime_Chi2_pT, sLegZprime.c_str(), "L");
	

	

	// vectors for the unbinned test:
	vector<Double_t> vec_REF_mHat;
	vector<Double_t> vec_REF_pT;
	
	vector<Double_t> vec_SM_mHat;
	vector<Double_t> vec_SM_pT;
	vector<Double_t> vec_Zprime_mHat;
	vector<Double_t> vec_Zprime_pT;
	vector<Double_t> vec_KK_mHat;
	vector<Double_t> vec_KK_pT;
	
	
	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons, Naleptons;
	double invariantMass, pTransverse;
	
	string treeType = "testTree_allPairs";
	
	// read the reference SM sample:
	if(nRefSelector==1) // SM ref
	{
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_839200_LargeRef_EventsSample.root");
	}
	else if(nRefSelector==2) // KK ref
	{
		if (mStarSelector==3) // m^* = 3 TeV
		{
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_793000_LargeRef_EventsSample.root");
		}
		else if (mStarSelector==4) // m^* = 4 TeV
		{
			filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_812900_LargeRef_EventsSample.root");
		}
	}
	Nevents = anlz.loopOverEvents(filePath, treeType.c_str(), 0, 0);
	cout << "Reference: Nevents = " << Nevents << endl;
	// get all the good pairs (after cuts)
	Nleptons = anlz.getAllLeptons(allgoodLeptons);
	Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
	cout << "Reference: Nleptons = " << Nleptons << endl;
	if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
	cout << "\n" << endl;
	for (int k=0 ; k<Nleptons ; k++)
	{
		invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
		pTransverse = allgoodLeptons[k].pT;
		// fill the histos
		REF_invMass->Fill( (Double_t)invariantMass );
		REF_mHat_statTests->Fill( (Double_t)invariantMass );
		REF_pT->Fill( (Double_t)pTransverse );
		REF_pT_statTests->Fill( (Double_t)pTransverse );
		
		vec_REF_mHat.push_back(invariantMass);
		vec_REF_pT.push_back(pTransverse);
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	if (allgoodLeptons.size()  != 0) { allgoodLeptons.clear(); }
	if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
	
	Int_t REF_Entries_mHat = (Int_t)vec_REF_mHat.size();
	Int_t REF_Entries_pT   = (Int_t)vec_REF_pT.size();
	Double_t *array_REF_mHat = (Double_t *)malloc(REF_Entries_mHat*sizeof(Double_t));
	Double_t *array_REF_pT   = (Double_t *)malloc(REF_Entries_pT*sizeof(Double_t));
	// sort the vectors:
	sort(vec_REF_mHat.begin(), vec_REF_mHat.end());
	sort(vec_REF_pT.begin(),   vec_REF_pT.end());
	// fill the sorted arrays:
	for (int jj=0 ; jj<(int)vec_REF_mHat.size() ; ++jj) { array_REF_mHat[jj] = vec_REF_mHat[jj]; }
	for (int jj=0 ; jj<(int)vec_REF_pT.size() ; ++jj)   { array_REF_pT[jj]   = vec_REF_pT[jj]; }
	
	
	const int nFiles   = 4;
	const int nSamples = 100;
	
	
	for(int nfile=1 ; nfile<=nFiles ; nfile++) // each file has 100 samples
	{
		// decide what file to read
		stringstream strm;
		string str = "";
		string sSuffix = "";
		strm << nfile;
		strm >> str;
		sSuffix = "_" + str + ".root";
		
		// initialize the starting point of each sample
		int nEventsToRead = 0;
		int nStart = 0;
		int nStart_SM=0;
		int nStart_KK=0;
		int nStart_E6=0;
		
		// now read the pseudo-data samples
		for(int sample=1 ; sample<=nSamples ; sample++) // run over the samples
		{
			// reset the histograms
			SM_invMass->Reset( "M" );
			SM_mHat_statTests->Reset( "M" );
			SM_pT->Reset( "M" );
			SM_pT_statTests->Reset( "M" );

			KK_invMass->Reset( "M" );
			KK_mHat_statTests->Reset( "M" );
			KK_pT->Reset( "M" );
			KK_pT_statTests->Reset( "M" );

			Zprime_invMass->Reset( "M" );
			Zprime_mHat_statTests->Reset( "M" );
			Zprime_pT->Reset( "M" );
			Zprime_pT_statTests->Reset( "M" );

			// reset the vectors used for the unbinned Kolmogorov test
			vec_SM_mHat.clear();
			vec_SM_pT.clear();

			vec_KK_mHat.clear();
			vec_KK_pT.clear();

			vec_Zprime_mHat.clear();
			vec_Zprime_pT.clear();

			// reset the vectors of the muons
			anlz.clearAllLeptons();
			allgoodLeptons.clear();
			allgoodaLeptons.clear();
			
			for (int f=1 ; f<=3 ; f++) // run over the 3 models files
			{
				if (f==1) { // SM
					if (mStarSelector==3) {
						nEventsToRead = 8392;
						nStart        = nStart_SM;
						nStart_SM    += nEventsToRead;
						filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_839200_EventsSample"+sSuffix);
					}
					else if(mStarSelector==4) {
						nEventsToRead = 8392;
						nStart        = nStart_SM;
						nStart_SM    += nEventsToRead;
						filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"SM_smeared_extractedParticles_2trees_839200_EventsSample"+sSuffix);
					}
				}
				else if (f==2) { // KK
					if (mStarSelector==3) {
						nEventsToRead = 7930;
						nStart        = nStart_KK;
						nStart_KK    += nEventsToRead;
						filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_793000_EventsSample"+sSuffix);
					}
					else if(mStarSelector==4) {
						nEventsToRead = 8129;
						nStart        = nStart_KK;
						nStart_KK    += nEventsToRead;
						filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"KK_smeared_extractedParticles_2trees_812900_EventsSample"+sSuffix);
					}
				}
				else if (f==3) { // E6
					if (mStarSelector==3) {
						nEventsToRead = 8338;
						nStart        = nStart_E6;
						nStart_E6    += nEventsToRead;
						filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_smeared_extractedParticles_2trees_833800_EventsSample"+sSuffix);
					}
					else if(mStarSelector==4) {
						nEventsToRead = 8361;
						nStart        = nStart_E6;
						nStart_E6    += nEventsToRead;
						filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"E6_smeared_extractedParticles_2trees_836100_EventsSample"+sSuffix);
					}
				}

				Nevents = anlz.loopOverEvents(filePath, treeType.c_str(), nEventsToRead, nStart);
				cout << "Nevents = " << Nevents << endl;
				// get all the good pairs (after cuts)
				Nleptons = anlz.getAllLeptons(allgoodLeptons);
				Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
				cout << "Nleptons = " << Nleptons << endl;
				if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
				cout << "\n" << endl;
				//-------------------------------------------------------------------------------------------------------------------
			
				// run over the good pairs and fill the histograms
				for (int k=0 ; k<Nleptons ; k++) {
					invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
					pTransverse = allgoodLeptons[k].pT;
					// fill the histos
					if (f==1) {
						SM_invMass->Fill( (Double_t)invariantMass );
						SM_mHat_statTests->Fill( (Double_t)invariantMass );
						SM_pT->Fill( (Double_t)pTransverse );
						SM_pT_statTests->Fill( (Double_t)pTransverse );

						vec_SM_mHat.push_back(invariantMass);
						vec_SM_pT.push_back(pTransverse);
					}
					else if (f==2) {
						KK_invMass->Fill( (Double_t)invariantMass );
						KK_mHat_statTests->Fill( (Double_t)invariantMass );
						KK_pT->Fill( (Double_t)pTransverse );
						KK_pT_statTests->Fill( (Double_t)pTransverse );
						
						vec_KK_mHat.push_back(invariantMass);
						vec_KK_pT.push_back(pTransverse);
					}
					else if (f==3) {
						Zprime_invMass->Fill( (Double_t)invariantMass );
						Zprime_mHat_statTests->Fill( (Double_t)invariantMass );
						Zprime_pT->Fill( (Double_t)pTransverse );
						Zprime_pT_statTests->Fill( (Double_t)pTransverse );

						vec_Zprime_mHat.push_back(invariantMass);
						vec_Zprime_pT.push_back(pTransverse);
					}
				}
				// clear the pairs vectors
				anlz.clearAllLeptons();
				if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
				if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
			}
			
			
			// chi2 and kolmogorov tests
			Double_t chi2 = 0.;
			Double_t kolmogorov = 0.;
			Double_t testMax = 0.99999;
			
			
			chi2       = SM_mHat_statTests->Chi2Test(REF_mHat_statTests, "UUOFUF");
			kolmogorov = SM_mHat_statTests->KolmogorovTest(REF_mHat_statTests, "UO");
			kolmogorov = (kolmogorov>=testMax) ? testMax : kolmogorov;
			chi2 = (chi2>=testMax) ? testMax : chi2;
			cout << "SM mHat histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;
			hSM_KprobBinned_mHat->Fill(kolmogorov);
			hSM_Chi2_mHat->Fill(chi2);
			
			chi2       = SM_pT_statTests->Chi2Test(REF_pT_statTests, "UUOFUF");
			kolmogorov = SM_pT_statTests->KolmogorovTest(REF_pT_statTests, "UO");
			kolmogorov = (kolmogorov>=testMax) ? testMax : kolmogorov;
			chi2 = (chi2>=testMax) ? testMax : chi2;
			cout << "SM pT histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;
			hSM_KprobBinned_pT->Fill(kolmogorov);
			hSM_Chi2_pT->Fill(chi2);
			
			
			chi2       = KK_mHat_statTests->Chi2Test(REF_mHat_statTests, "UUOFUF");
			kolmogorov = KK_mHat_statTests->KolmogorovTest(REF_mHat_statTests, "UO");
			kolmogorov = (kolmogorov>=testMax) ? testMax : kolmogorov;
			chi2 = (chi2>=testMax) ? testMax : chi2;
			cout << "KK mHat histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;
			hKK_KprobBinned_mHat->Fill(kolmogorov);
			hKK_Chi2_mHat->Fill(chi2);

			chi2       = KK_pT_statTests->Chi2Test(REF_pT_statTests, "UUOFUF");
			kolmogorov = KK_pT_statTests->KolmogorovTest(REF_pT_statTests, "UO");
			kolmogorov = (kolmogorov>=testMax) ? testMax : kolmogorov;
			chi2 = (chi2>=testMax) ? testMax : chi2;
			cout << "KK pT histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;
			hKK_KprobBinned_pT->Fill(kolmogorov);
			hKK_Chi2_pT->Fill(chi2);	

			
			chi2       = Zprime_mHat_statTests->Chi2Test(REF_mHat_statTests, "UUOFUF");
			kolmogorov = Zprime_mHat_statTests->KolmogorovTest(REF_mHat_statTests, "UO");
			kolmogorov = (kolmogorov>=testMax) ? testMax : kolmogorov;
			chi2 = (chi2>=testMax) ? testMax : chi2;
			cout << "Z' mHat histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;
			hZprime_KprobBinned_mHat->Fill(kolmogorov);
			hZprime_Chi2_mHat->Fill(chi2);

			chi2       = Zprime_pT_statTests->Chi2Test(REF_pT_statTests, "UUOFUF");
			kolmogorov = Zprime_pT_statTests->KolmogorovTest(REF_pT_statTests, "UO");
			kolmogorov = (kolmogorov>=testMax) ? testMax : kolmogorov;
			chi2 = (chi2>=testMax) ? testMax : chi2;
			cout << "Z' pT histo:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;
			hZprime_KprobBinned_pT->Fill(kolmogorov);
			hZprime_Chi2_pT->Fill(chi2);
			
			
			// prepare for the unbinned Kolmogorov test
			Int_t SM_Entries_mHat = (Int_t)vec_SM_mHat.size();
			Int_t SM_Entries_pT = (Int_t)vec_SM_pT.size();
			Double_t *array_SM_mHat = (Double_t *)malloc(SM_Entries_mHat*sizeof(Double_t));
			Double_t *array_SM_pT = (Double_t *)malloc(SM_Entries_pT*sizeof(Double_t));
			// sort the vectors:
			sort(vec_SM_mHat.begin(), vec_SM_mHat.end());
			sort(vec_SM_pT.begin(), vec_SM_pT.end());
			// fill the sorted arrays:
			for (int j=0 ; j<(int)vec_SM_mHat.size() ; ++j) { array_SM_mHat[j] = vec_SM_mHat[j]; }
			for (int j=0 ; j<(int)vec_SM_pT.size() ; ++j) { array_SM_pT[j] = vec_SM_pT[j]; }

			Int_t KK_Entries_mHat = (Int_t)vec_KK_mHat.size();
			Int_t KK_Entries_pT = (Int_t)vec_KK_pT.size();
			Double_t *array_KK_mHat = (Double_t *)malloc(KK_Entries_mHat*sizeof(Double_t));
			Double_t *array_KK_pT = (Double_t *)malloc(KK_Entries_pT*sizeof(Double_t));
			// sort the vectors:
			sort(vec_KK_mHat.begin(), vec_KK_mHat.end());
			sort(vec_KK_pT.begin(), vec_KK_pT.end());
			// fill the sorted arrays:
			for (int j=0 ; j<(int)vec_KK_mHat.size() ; ++j) { array_KK_mHat[j] = vec_KK_mHat[j]; }
			for (int j=0 ; j<(int)vec_KK_pT.size() ; ++j) { array_KK_pT[j] = vec_KK_pT[j]; }

			Int_t Zprime_Entries_mHat = (Int_t)vec_Zprime_mHat.size();
			Int_t Zprime_Entries_pT = (Int_t)vec_Zprime_pT.size();
			Double_t *array_Zprime_mHat = (Double_t *)malloc(Zprime_Entries_mHat*sizeof(Double_t));
			Double_t *array_Zprime_pT = (Double_t *)malloc(Zprime_Entries_pT*sizeof(Double_t));
			// sort the vectors:
			sort(vec_Zprime_mHat.begin(), vec_Zprime_mHat.end());
			sort(vec_Zprime_pT.begin(), vec_Zprime_pT.end());
			// fill the sorted arrays:
			for (int j=0 ; j<(int)vec_Zprime_mHat.size() ; ++j) { array_Zprime_mHat[j] = vec_Zprime_mHat[j]; }
			for (int j=0 ; j<(int)vec_Zprime_pT.size() ; ++j) { array_Zprime_pT[j] = vec_Zprime_pT[j]; }

			// do the unbinned Kolmogorov test:
			Double_t unbinnedKStest_SM_mHat     = TMath::KolmogorovTest((Int_t)vec_REF_mHat.size(), array_REF_mHat, (Int_t)vec_SM_mHat.size(), array_SM_mHat ,"");
			Double_t unbinnedKStest_KK_mHat     = TMath::KolmogorovTest((Int_t)vec_REF_mHat.size(), array_REF_mHat, (Int_t)vec_KK_mHat.size(), array_KK_mHat ,"");
			Double_t unbinnedKStest_Zprime_mHat = TMath::KolmogorovTest((Int_t)vec_REF_mHat.size(), array_REF_mHat, (Int_t)vec_Zprime_mHat.size(), array_Zprime_mHat ,"");

			Double_t unbinnedKStest_SM_pT     = TMath::KolmogorovTest((Int_t)vec_REF_pT.size(), array_REF_pT, (Int_t)vec_SM_pT.size(), array_SM_pT ,"");
			Double_t unbinnedKStest_KK_pT     = TMath::KolmogorovTest((Int_t)vec_REF_pT.size(), array_REF_pT, (Int_t)vec_KK_pT.size(), array_KK_pT ,"");
			Double_t unbinnedKStest_Zprime_pT = TMath::KolmogorovTest((Int_t)vec_REF_pT.size(), array_REF_pT, (Int_t)vec_Zprime_pT.size(), array_Zprime_pT ,"");

			cout << "SM: mHat unbinned KStest = " << unbinnedKStest_SM_mHat << "\t" << "pT unbinned KStest = " << unbinnedKStest_SM_pT << endl;
			hSM_KprobUnbinned_mHat->Fill(unbinnedKStest_SM_mHat);
			hSM_KprobUnbinned_pT->Fill(unbinnedKStest_SM_pT);
			cout << "KK: mHat unbinned KStest = " << unbinnedKStest_KK_mHat << "\t" << "pT unbinned KStest = " << unbinnedKStest_KK_pT << endl;
			hKK_KprobUnbinned_mHat->Fill(unbinnedKStest_KK_mHat);
			hKK_KprobUnbinned_pT->Fill(unbinnedKStest_KK_pT);
			cout << "Z': mHat unbinned KStest = " << unbinnedKStest_Zprime_mHat << "\t" << "pT unbinned KStest = " << unbinnedKStest_Zprime_pT << endl;
			hZprime_KprobUnbinned_mHat->Fill(unbinnedKStest_Zprime_mHat);
			hZprime_KprobUnbinned_pT->Fill(unbinnedKStest_Zprime_pT);
		}
	}
	cout << "\n" << endl;
	
	// plot the histograms and write the .root + .eps files
	Double_t max = 0.;
	double confidenceLevel = 95.;
	double prob = 0.;

	
	dir_mHat_KprobBinned->cd();
	canv_mHat_KprobBinned->Draw();
	canv_mHat_KprobBinned->cd();
	max = (hSM_KprobBinned_mHat->GetMaximum()>hKK_KprobBinned_mHat->GetMaximum()) ? 1.1*hSM_KprobBinned_mHat->GetMaximum() : 1.1*hKK_KprobBinned_mHat->GetMaximum();
	max = (hZprime_KprobBinned_mHat->GetMaximum()>max) ? 1.1*hZprime_KprobBinned_mHat->GetMaximum() : max;
	hSM_KprobBinned_mHat->SetMaximum(max);
	hSM_KprobBinned_mHat->SetMinimum(0.);
	hSM_KprobBinned_mHat->Draw();
	hZprime_KprobBinned_mHat->Draw("sames");
	hKK_KprobBinned_mHat->Draw("sames");
	leg_mHat_KprobBinned->Draw("sames");
	//pave_mHat_KprobBinned.Draw("sames");
	canv_mHat_KprobBinned->Update();
	canv_mHat_KprobBinned->Write();
	/*
	if (nRefSelector==1) // SM ref
	{
		prob = commonOps.getProbability(hSM_KprobBinned_mHat, hZprime_KprobBinned_mHat, confidenceLevel);
		cout << "KprobBinned mHat:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_KprobBinned_mHat, confidenceLevel);
		cout << "KprobBinned mHat:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hSM_KprobBinned_mHat, hKK_KprobBinned_mHat, confidenceLevel);
		cout << "KprobBinned mHat:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hKK_KprobBinned_mHat, confidenceLevel);
		cout << "KprobBinned mHat:  p(KK ; from flat) = " << prob << endl;
	}
	else if (nRefSelector==2) // KK ref
	{
		prob = commonOps.getProbability(hKK_KprobBinned_mHat, hZprime_KprobBinned_mHat, confidenceLevel);
		cout << "KprobBinned mHat:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_KprobBinned_mHat, confidenceLevel);
		cout << "KprobBinned mHat:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hKK_KprobBinned_mHat, hSM_KprobBinned_mHat, confidenceLevel);
		cout << "KprobBinned mHat:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hSM_KprobBinned_mHat, confidenceLevel);
		cout << "KprobBinned mHat:  p(KK ; from flat) = " << prob << endl;
	}
	cout << endl;
	*/
	

	dir_mHat_KprobUnbinned->cd();
	canv_mHat_KprobUnbinned->Draw();
	canv_mHat_KprobUnbinned->cd();
	max = (hSM_KprobUnbinned_mHat->GetMaximum()>hKK_KprobUnbinned_mHat->GetMaximum()) ? 1.1*hSM_KprobUnbinned_mHat->GetMaximum() : 1.1*hKK_KprobUnbinned_mHat->GetMaximum();
	max = (hZprime_KprobUnbinned_mHat->GetMaximum()>max) ? 1.1*hZprime_KprobUnbinned_mHat->GetMaximum() : max;
	hSM_KprobUnbinned_mHat->SetMaximum(max);
	hSM_KprobUnbinned_mHat->SetMinimum(0.);
	hSM_KprobUnbinned_mHat->Draw();
	hZprime_KprobUnbinned_mHat->Draw("sames");
	hKK_KprobUnbinned_mHat->Draw("sames");
	leg_mHat_KprobUnbinned->Draw("sames");
	//pave_mHat_KprobUnbinned.Draw("sames");
	canv_mHat_KprobUnbinned->Update();
	canv_mHat_KprobUnbinned->Write();
	if (nRefSelector==1) // SM ref
	{
		prob = commonOps.getProbability(hSM_KprobUnbinned_mHat, hZprime_KprobUnbinned_mHat, confidenceLevel);
		cout << "KprobUnbinned mHat:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_KprobUnbinned_mHat, confidenceLevel);
		cout << "KprobUnbinned mHat:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hSM_KprobUnbinned_mHat, hKK_KprobUnbinned_mHat, confidenceLevel);
		cout << "KprobUnbinned mHat:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hKK_KprobUnbinned_mHat, confidenceLevel);
		cout << "KprobUnbinned mHat:  p(KK ; from flat) = " << prob << endl;
	}
	else if (nRefSelector==2) // KK ref
	{
		prob = commonOps.getProbability(hKK_KprobUnbinned_mHat, hZprime_KprobUnbinned_mHat, confidenceLevel);
		cout << "KprobUnbinned mHat:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_KprobUnbinned_mHat, confidenceLevel);
		cout << "KprobUnbinned mHat:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hKK_KprobUnbinned_mHat, hSM_KprobUnbinned_mHat, confidenceLevel);
		cout << "KprobUnbinned mHat:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hSM_KprobUnbinned_mHat, confidenceLevel);
		cout << "KprobUnbinned mHat:  p(KK ; from flat) = " << prob << endl;
	}
	cout << endl;
	
	dir_mHat_Chi2->cd();
	canv_mHat_Chi2->Draw();
	canv_mHat_Chi2->cd();
	max = (hSM_Chi2_mHat->GetMaximum()>hKK_Chi2_mHat->GetMaximum()) ? 1.1*hSM_Chi2_mHat->GetMaximum() : 1.1*hKK_Chi2_mHat->GetMaximum();
	max = (hZprime_Chi2_mHat->GetMaximum()>max) ? 1.1*hZprime_Chi2_mHat->GetMaximum() : max;
	hSM_Chi2_mHat->SetMaximum(max);
	hSM_Chi2_mHat->SetMinimum(0.);
	hSM_Chi2_mHat->Draw();
	hZprime_Chi2_mHat->Draw("sames");
	hKK_Chi2_mHat->Draw("sames");
	leg_mHat_Chi2->Draw("sames");
	//pave_mHat_Chi2.Draw("sames");
	canv_mHat_Chi2->Update();
	canv_mHat_Chi2->Write();
	/*
	if (nRefSelector==1) // SM ref
	{
		prob = commonOps.getProbability(hSM_Chi2_mHat, hZprime_Chi2_mHat, confidenceLevel);
		cout << "Chi2 mHat:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_Chi2_mHat, confidenceLevel);
		cout << "Chi2 mHat:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hSM_Chi2_mHat, hKK_Chi2_mHat, confidenceLevel);
		cout << "Chi2 mHat:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hKK_Chi2_mHat, confidenceLevel);
		cout << "Chi2 mHat:  p(KK ; from flat) = " << prob << endl;
	}
	else if (nRefSelector==2) // KK ref
	{
		prob = commonOps.getProbability(hKK_Chi2_mHat, hZprime_Chi2_mHat, confidenceLevel);
		cout << "Chi2 mHat:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_Chi2_mHat, confidenceLevel);
		cout << "Chi2 mHat:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hKK_Chi2_mHat, hSM_Chi2_mHat, confidenceLevel);
		cout << "Chi2 mHat:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hSM_Chi2_mHat, confidenceLevel);
		cout << "Chi2 mHat:  p(KK ; from flat) = " << prob << endl;
	}
	cout << endl;
	*/
	
	
	
	
	dir_pT_KprobBinned->cd();
	canv_pT_KprobBinned->Draw();
	canv_pT_KprobBinned->cd();
	max = (hSM_KprobBinned_pT->GetMaximum()>hKK_KprobBinned_pT->GetMaximum()) ? 1.1*hSM_KprobBinned_pT->GetMaximum() : 1.1*hKK_KprobBinned_pT->GetMaximum();
	max = (hZprime_KprobBinned_pT->GetMaximum()>max) ? 1.1*hZprime_KprobBinned_pT->GetMaximum() : max;
	hSM_KprobBinned_pT->SetMaximum(max);
	hSM_KprobBinned_pT->SetMinimum(0.);
	hSM_KprobBinned_pT->Draw();
	hZprime_KprobBinned_pT->Draw("sames");
	hKK_KprobBinned_pT->Draw("sames");
	leg_pT_KprobBinned->Draw("sames");
	//pave_pT_KprobBinned.Draw("sames");
	canv_pT_KprobBinned->Update();
	canv_pT_KprobBinned->Write();
	/*
	if (nRefSelector==1) // SM ref
	{
		prob = commonOps.getProbability(hSM_KprobBinned_pT, hZprime_KprobBinned_pT, confidenceLevel);
		cout << "KprobBinned pT:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_KprobBinned_pT, confidenceLevel);
		cout << "KprobBinned pT:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hSM_KprobBinned_pT, hKK_KprobBinned_pT, confidenceLevel);
		cout << "KprobBinned pT:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hKK_KprobBinned_pT, confidenceLevel);
		cout << "KprobBinned pT:  p(KK ; from flat) = " << prob << endl;
	}
	else if (nRefSelector==2) // KK ref
	{
		prob = commonOps.getProbability(hKK_KprobBinned_pT, hZprime_KprobBinned_pT, confidenceLevel);
		cout << "KprobBinned pT:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_KprobBinned_pT, confidenceLevel);
		cout << "KprobBinned pT:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hKK_KprobBinned_pT, hSM_KprobBinned_pT, confidenceLevel);
		cout << "KprobBinned pT:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hSM_KprobBinned_pT, confidenceLevel);
		cout << "KprobBinned pT:  p(KK ; from flat) = " << prob << endl;
	}
	cout << endl;
	*/
	
	dir_pT_KprobUnbinned->cd();
	canv_pT_KprobUnbinned->Draw();
	canv_pT_KprobUnbinned->cd();
	max = (hSM_KprobUnbinned_pT->GetMaximum()>hKK_KprobUnbinned_pT->GetMaximum()) ? 1.1*hSM_KprobUnbinned_pT->GetMaximum() : 1.1*hKK_KprobUnbinned_pT->GetMaximum();
	max = (hZprime_KprobUnbinned_pT->GetMaximum()>max) ? 1.1*hZprime_KprobUnbinned_pT->GetMaximum() : max;
	hSM_KprobUnbinned_pT->SetMaximum(max);
	hSM_KprobUnbinned_pT->SetMinimum(0.);
	hSM_KprobUnbinned_pT->Draw();
	hZprime_KprobUnbinned_pT->Draw("sames");
	hKK_KprobUnbinned_pT->Draw("sames");
	leg_pT_KprobUnbinned->Draw("sames");
	//pave_pT_KprobUnbinned.Draw("sames");
	canv_pT_KprobUnbinned->Update();
	canv_pT_KprobUnbinned->Write();
	if (nRefSelector==1) // SM ref
	{
		prob = commonOps.getProbability(hSM_KprobUnbinned_pT, hZprime_KprobUnbinned_pT, confidenceLevel);
		cout << "KprobUnbinned pT:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_KprobUnbinned_pT, confidenceLevel);
		cout << "KprobUnbinned pT:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hSM_KprobUnbinned_pT, hKK_KprobUnbinned_pT, confidenceLevel);
		cout << "KprobUnbinned pT:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hKK_KprobUnbinned_pT, confidenceLevel);
		cout << "KprobUnbinned pT:  p(KK ; from flat) = " << prob << endl;
	}
	else if (nRefSelector==2) // KK ref
	{
		prob = commonOps.getProbability(hKK_KprobUnbinned_pT, hZprime_KprobUnbinned_pT, confidenceLevel);
		cout << "KprobUnbinned pT:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_KprobUnbinned_pT, confidenceLevel);
		cout << "KprobUnbinned pT:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hKK_KprobUnbinned_pT, hSM_KprobUnbinned_pT, confidenceLevel);
		cout << "KprobUnbinned pT:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hSM_KprobUnbinned_pT, confidenceLevel);
		cout << "KprobUnbinned pT:  p(KK ; from flat) = " << prob << endl;
	}
	cout << endl;
	
	dir_pT_Chi2->cd();
	canv_pT_Chi2->Draw();
	canv_pT_Chi2->cd();
	max = (hSM_Chi2_pT->GetMaximum()>hKK_Chi2_pT->GetMaximum()) ? 1.1*hSM_Chi2_pT->GetMaximum() : 1.1*hKK_Chi2_pT->GetMaximum();
	max = (hZprime_Chi2_pT->GetMaximum()>max) ? 1.1*hZprime_Chi2_pT->GetMaximum() : max;
	hSM_Chi2_pT->SetMaximum(max);
	hSM_Chi2_pT->SetMinimum(0.);
	hSM_Chi2_pT->Draw();
	hZprime_Chi2_pT->Draw("sames");
	hKK_Chi2_pT->Draw("sames");
	leg_pT_Chi2->Draw("sames");
	//pave_pT_Chi2.Draw("sames");
	canv_pT_Chi2->Update();
	canv_pT_Chi2->Write();
	/*
	if (nRefSelector==1) // SM ref
	{
		prob = commonOps.getProbability(hSM_Chi2_pT, hZprime_Chi2_pT, confidenceLevel);
		cout << "Chi2 pT:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_Chi2_pT, confidenceLevel);
		cout << "Chi2 pT:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hSM_Chi2_pT, hKK_Chi2_pT, confidenceLevel);
		cout << "Chi2 pT:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hKK_Chi2_pT, confidenceLevel);
		cout << "Chi2 pT:  p(KK ; from flat) = " << prob << endl;
	}
	else if (nRefSelector==2) // KK ref
	{
		prob = commonOps.getProbability(hKK_Chi2_pT, hZprime_Chi2_pT, confidenceLevel);
		cout << "Chi2 pT:  p(Z' ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hZprime_Chi2_pT, confidenceLevel);
		cout << "Chi2 pT:  p(Z' ; from flat) = " << prob << endl;
		prob = commonOps.getProbability(hKK_Chi2_pT, hSM_Chi2_pT, confidenceLevel);
		cout << "Chi2 pT:  p(KK ; from ref)  = " << prob << endl;
		prob = commonOps.getProbability(hSM_Chi2_pT, confidenceLevel);
		cout << "Chi2 pT:  p(KK ; from flat) = " << prob << endl;
	}
	cout << endl;
	*/

	
	statTestsHistFile->Write();
	
	
	canv_mHat_KprobBinned->Print(epsFileName_mHatKprobBinned.c_str());
	canv_mHat_KprobUnbinned->Print(epsFileName_mHatKprobUnbinned.c_str());
	canv_mHat_Chi2->Print(epsFileName_mHatChi2.c_str());
	canv_pT_KprobBinned->Print(epsFileName_pTKprobBinned.c_str());
	canv_pT_KprobUnbinned->Print(epsFileName_pTKprobUnbinned.c_str());
	canv_pT_Chi2->Print(epsFileName_pTChi2.c_str());
	

	canv_mHat_KprobBinned->WaitPrimitive();
	canv_mHat_KprobUnbinned->WaitPrimitive();
	canv_mHat_Chi2->WaitPrimitive();
	canv_pT_KprobBinned->WaitPrimitive();
	canv_pT_KprobUnbinned->WaitPrimitive();
	canv_pT_Chi2->WaitPrimitive();

	// Done.
	return 0;
}

