/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Moses1;
using namespace Pythia8;


//TH1 *GLOBAL_eff = new TH1D("", "", GLOBAL_BINS, -1., 1.);
int    GLOBAL_BINS = 100;
double *GLOBAL_eff = (double *)malloc(GLOBAL_BINS*sizeof(double));
double GLOBAL_cos2thetaAverage = 1.;
double GLOBAL_costhetaAverage  = 1.;
double GLOBAL_efficiencySum    = 1.;
double GLOBAL_numberOfEvents   = 1.;
double GLOBAL_absRange         = 1.;
double GLOBAL_scale	       = 1.;
int    GLOBAL_luminosity       = 1;

// Declare pointer to data as global (not elegant but TMinuit needs this).
vector<double> *GLOBAL_cosTheta = new vector<double>();



// The pdf to be fitted
// First argument needs to be a pointer in order to plot with the TF1 class.
double unBiasedCollinsSoperPdf(double* xPtr, double par[]){

        double x  = *xPtr;
        double dx = 2.*GLOBAL_absRange/(double)GLOBAL_BINS;
        //cout << "x = " << x << endl;

        double collinsSoper     = 1.;
        double efficiency       = 1.;
        double normalization    = 1.;
        double fullDistribution = 1.;
	double histoIntegral	= 1.;
        int    binLocationIndex = 0;

	double A0 = 0.;
	double A4 = 0.;

	if (GLOBAL_luminosity==1) {
		A0 = par[0];
		A4 = par[1];
	}
	else {
		A4 = par[0]; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	}
	

        if (x!=GLOBAL_absRange) { binLocationIndex = (int)((x+GLOBAL_absRange)/dx); }
        else                    { binLocationIndex = GLOBAL_BINS-1; }

        if (abs(x) <= GLOBAL_absRange) {
		collinsSoper  = 1.+0.5*A0 + A4*x + (1.-1.5*A0)*x*x;
		histoIntegral = GLOBAL_numberOfEvents * dx;

		if (GLOBAL_absRange<1.) {
			normalization = (1.+0.5*A0)*2.*GLOBAL_absRange + (1.-1.5*A0)*2.*GLOBAL_absRange*GLOBAL_absRange*GLOBAL_absRange/3.;
			fullDistribution = (collinsSoper / normalization) * (histoIntegral);
		}
		else if (GLOBAL_absRange==1.) {
			efficiency       = GLOBAL_eff[binLocationIndex];
			normalization    = GLOBAL_efficiencySum * ( 1.+0.5*A0 + A4*GLOBAL_costhetaAverage + (1.-1.5*A0)*GLOBAL_cos2thetaAverage );
			fullDistribution = ( (efficiency * collinsSoper) / normalization ) * (histoIntegral);
		}
		if (fullDistribution > 0) { return fullDistribution; }
		else { cout << "warning:  fullDistribution<=0, fullDistribution=" << fullDistribution << endl; return 1e-30; }
        }
        else { cout << "warning:  x<-1 || x>+1, x=" << x << endl; return 1e-30; }
}

// as above only scaled
double scaledFunction(double* xPtr, double par[]){
	return GLOBAL_scale*unBiasedCollinsSoperPdf(xPtr, par);
}

// fcn passes back f = - 2*ln(L), the function to be minimized.
void fcn(int& npar, double* deriv, double& f, double par[], int flag) {
        vector<double> xVec = *GLOBAL_cosTheta; // GLOBAL_cosTheta is global
        int n = xVec.size();
        double lnL = 0.;
        double x   = 0.;
        double pdf = 0.;
        for (int i=0; i<n; i++){
		x = xVec[i];
		pdf = unBiasedCollinsSoperPdf(&x, par);
		if (pdf > 0.) {
			lnL += log(pdf); // need positive f
		}
		else { cout << "WARNING -- pdf is negative!!!" << endl; }
        }
        f = -2.0 * lnL; // factor of -2 so minuit gets the errors right
} // end of fcn



int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
	string dirName = "/KKvsZE6vsSMstudy/";
	int iModel = 1;
	string sModel = "KK";
	cout << "KK(1), SM(2) or E6(3) ?" << endl;
	cout << "..." << endl;
	cin >> iModel;
	switch(iModel) {
		case 1: sModel = "KK"; break;
		case 2: sModel = "SM"; break;
		case 3: sModel = "E6"; break;
		default: cout << "wrong option, set to default: KK(1)" << endl;
			 sModel = "KK"; break;
			 break;
	}
	string outFileName = dirName + "TMinuit/" + sModel + "_TMinuit_collinsSoper";


	int sampleSizeSelector = 1;
	cout << "select sample size: 1M(1), L=500fb^-1(2), L=100fb^-1(3)..." << endl;
	cin >> sampleSizeSelector;
	if (sampleSizeSelector!=1 && sampleSizeSelector!=2 && sampleSizeSelector!=3) {
		cout << "invalid option, set to 1 (default)" << endl;
		sampleSizeSelector = 1;
	}
	GLOBAL_luminosity = sampleSizeSelector;
	string sSampleSize = "";
	switch (sampleSizeSelector) {
		case 1: sSampleSize = "_LrefinvFb"; break;
		case 2: sSampleSize = "_L500invFb"; break;
		case 3: sSampleSize = "_L100invFb"; break;
		default: break;
	}
	outFileName += sSampleSize;



	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	Particle particle;

	// set cuts
	double invMassCut=1000.;
	double pTransverseCut=10.;
	double EtaCut=2.5;

	double no_invMassCut=0.;
	double no_pTransverseCut=0.;
	double no_EtaCut=1e10;


	// choose reference frame
	int referenceFrameSelector = 0;
	string cosThetaTitle;
	cout << "choose reference frame to calculate cos(theta)" << endl;
	cout << "(1) Helicity frame or (2) Collins-Soper frame" << endl;
	cout << "..." << endl;
	cin >> referenceFrameSelector;
	if (referenceFrameSelector != 1 && referenceFrameSelector != 2) {
		cout << "invalid option, setting to default = 1" << endl;
		referenceFrameSelector = 1;
	}
	if (referenceFrameSelector == 1) {cosThetaTitle = "Helicity";}
	else if (referenceFrameSelector == 2) {cosThetaTitle = "Collins-Soper";}


	int sampleSelector = 1;
	string withORwithoutCuts;
	cout << "select sample type: withCuts(1), noCuts(2)..." << endl;
	cin >> sampleSelector;
	if (sampleSelector!=1 && sampleSelector!=2) {
		cout << "wrong option: sampleSelector=" << sampleSelector << " setting to default (1)" << endl;
		sampleSelector = 1;
	}
	else {
		switch(sampleSelector) {
			case 1: withORwithoutCuts = "_withCuts"; break;
			case 2: withORwithoutCuts = "_noCuts"; break;
			default: break;
		}
	}
	//--- select static method of calculating costheta
	int quarkDirectionOrRapidity = 2; // 1=quark direction, 2=rapidity sign


	double absRange = 1.;
	cout << "enter abs fit range <=1..." << endl;
	cin >> absRange;
	if (absRange>1 || absRange<0) {
		cout << "wrong option: absRange=" << absRange << " setting to default 1" << endl;
		absRange = 1.;
	}
	GLOBAL_absRange = absRange;
	string strFitRange = "";
	stringstream sstrmFitRange;
	sstrmFitRange << absRange;
	sstrmFitRange >> strFitRange;
	outFileName += "_fitAbsRange" + strFitRange;


	// declaration of histograms
	//---------------------------------------------------------------------------------------------------------
	double cosThetamin   = -1.;
	double cosThetamax   = 1.;

	// titles for the cos(theta) plots
	string title  = "";
	string xtitle = "cos#theta'";
	string ytitle = "#frac{dN}{dcos#theta'}";

	TH1 *cosTheta_withCuts = new TH1D("", "", GLOBAL_BINS, -GLOBAL_absRange, GLOBAL_absRange);
	TH1 *cosTheta_noCuts = new TH1D("", "", GLOBAL_BINS, -GLOBAL_absRange, GLOBAL_absRange);
	
	cosTheta_withCuts->SetLineStyle(1);
	cosTheta_withCuts->SetMarkerStyle(20);
	cosTheta_withCuts->SetXTitle(xtitle.c_str());
	cosTheta_withCuts->SetYTitle(ytitle.c_str());

	cosTheta_noCuts->SetLineStyle(1);
	cosTheta_noCuts->SetMarkerStyle(20);
	cosTheta_noCuts->SetXTitle(xtitle.c_str());
	cosTheta_noCuts->SetYTitle(ytitle.c_str());
	//----------------------------------------------------------------------------------------------------------


	// declare kinematics histograms and root items
	string TMinuit_collinssoperRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + withORwithoutCuts + ".root");
	string TMinuit_collinssoperEpsFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + withORwithoutCuts + ".eps");


	TFile *TMinuit_collinssoperHistFile = new TFile(TMinuit_collinssoperRootFileName.c_str(),"recreate");
	TCanvas *canv = new TCanvas("canv","Kaluza-Klein TMinuit collins soper",0,0,1400,800);


	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons, Naleptons;
	double cosThetaBoost, cosThetaCollinsSoper;
	double invariantMass, cosTh=0;

	// to check the asymmetry in a narrow range arrownd the peak
	double mZ_known = physPrm.mStar;
	double mZ_lowLimit = physPrm.mStar-2000.;
	double mZ_highLimit = physPrm.mStar+1000.;
	cout << "\n" << "mZ_known=" << mZ_known << "  mZ_lowLimit=" << mZ_lowLimit << "  mZ_highLimit=" << mZ_highLimit << "\n" << endl;
	


	// read the trees of the reference sample and put all good lepton pairs into 2 vectors
	filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+sModel+"_extractedParticles_2trees_1Msample"+".root");
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);
	Nevents = anlz.loopOverEvents(filePath,"testTree",0,0);
	cout << "Nevents = " << Nevents << endl;
	// get all the good pairs (after cuts)
	Nleptons = anlz.getAllLeptons(allgoodLeptons);
	Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
	cout << "Nleptons = " << Nleptons << endl;
	if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
	cout << "\n" << endl;
	
	Int_t nEvents_withCuts_allRange = 0;
	// run over the good pairs (passed cuts) and fill the histograms
	for (int k=0 ; k<Nleptons ; k++) {
		invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
		if (quarkDirectionOrRapidity==2) {
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
		}
		else if (quarkDirectionOrRapidity==1) {
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k], true);
		}
		else {cout << "wrong option for quarkDirectionOrRapidity: " << quarkDirectionOrRapidity << " exitting now" << endl; exit(1);}
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the histos
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			if (abs(cosTh)<=GLOBAL_absRange) {
				cosTheta_withCuts->Fill( cosTh );
			}
			nEvents_withCuts_allRange ++;
		}
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
	if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }	
		
	//-------------------------------------------------------------------------------------------------------------------
	
	// no cuts
	Int_t nEvents_noCuts_allRange = 0;
	anlz.setCuts(no_invMassCut, no_pTransverseCut, no_EtaCut);
	Nevents = anlz.loopOverEvents(filePath,"testTree_allPairs", 0,0);
	cout << "Nevents_allPairs = " << Nevents << endl;
	// get all the good pairs (no cuts)
	Nleptons = anlz.getAllLeptons(allgoodLeptons);
	Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
	cout << "Nleptons_allPairs = " << Nleptons << endl;
	if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
	cout << "\n" << endl;

	// run over the good pairs (passed cuts) and fill the histograms
	for (int k=0 ; k<Nleptons ; k++) {
		invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
		cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
		if (quarkDirectionOrRapidity==2) {
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
		}
		else if (quarkDirectionOrRapidity==1) {
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k], true);
		}
		if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
		else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
		// fill the histos
		if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			if (abs(cosTh)<=GLOBAL_absRange) {
				cosTheta_noCuts->Fill( cosTh );
			}
			nEvents_noCuts_allRange ++;
		}
	}
	// clear the pairs vectors
	anlz.clearAllLeptons();
	if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
	if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }

	cout << "the number of entries of the reference sample in the fit range +-1" << endl;
        cout << "nEvents_withCuts_allRange = " << nEvents_withCuts_allRange << endl;
        cout << "nEvents_noCuts_allRange = " << nEvents_noCuts_allRange << "\n" << endl;

	cout << "the number of entries of the reference sample in the fit range +-" << strFitRange << endl;
	cout << "cosTheta_withCuts = " << cosTheta_withCuts->GetEntries() << endl;
	cout << "cosTheta_noCuts = " << cosTheta_noCuts->GetEntries() << endl;
	
	
	if      (sampleSelector==1) { GLOBAL_numberOfEvents = (int)cosTheta_withCuts->GetEntries(); }
	else if (sampleSelector==2) { GLOBAL_numberOfEvents = (int)cosTheta_noCuts->GetEntries(); }


	// constants
	GLOBAL_efficiencySum    = 0.; // initialize
	GLOBAL_cos2thetaAverage = 0.; // initialize
	GLOBAL_costhetaAverage  = 0.; // initialize
	double cosTheta, ratio, withCuts=0., noCuts=0.;
	double dCost = 2.*GLOBAL_absRange/(double)GLOBAL_BINS;
	for (int n=0 ; n<GLOBAL_BINS ; n++) {
		cosTheta = -GLOBAL_absRange + (1.+2*n)*0.5*dCost;
		withCuts = cosTheta_withCuts->GetBinContent(n+1);
		noCuts   = cosTheta_noCuts->GetBinContent(n+1);
		if (noCuts!=0) {
			ratio = withCuts/noCuts;
			GLOBAL_efficiencySum += ratio;
		}
		else { ratio = 1e-10; }
		if (sampleSelector==2) { // no cuts
			ratio = 1.;
		}
		GLOBAL_eff[n] = ratio;
		GLOBAL_cos2thetaAverage += cosTheta*cosTheta * ratio;
		GLOBAL_costhetaAverage  += cosTheta * ratio;
	}
	GLOBAL_cos2thetaAverage /= GLOBAL_efficiencySum;
	GLOBAL_costhetaAverage  /= GLOBAL_efficiencySum;
	GLOBAL_efficiencySum    *= dCost;
	if (sampleSelector==2) { // no cuts
		GLOBAL_efficiencySum    = 2.;
		GLOBAL_cos2thetaAverage = 1./3.;
		GLOBAL_costhetaAverage  = 0.;
	}
	cout	<< "GLOBAL_costhetaAverage=" << GLOBAL_costhetaAverage << "\t"
		<< "GLOBAL_cos2thetaAverage=" << GLOBAL_cos2thetaAverage << "\t"
		<< "GLOBAL_efficiencySum=" << GLOBAL_efficiencySum << "\t"
		<< "GLOBAL_numberOfEvents=" << GLOBAL_numberOfEvents << endl;





	//--- select sample type and read it
	// read the trees and put all good lepton pairs into 2 vectors
	if (sampleSizeSelector==1) { // sample size = 1M
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+sModel+"_extractedParticles_2trees_1Msample"+".root");
	}
	else if (sampleSizeSelector==2) { // L=500fb^-1
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+sModel+"_extractedParticles_2trees_500inversefemtobarnSample.root");
	}
	else if (sampleSizeSelector==3) { // L=100fb^-1
		filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100inversefemtobarn/"+sModel+"_extractedParticles_2trees_100inversefemtobarnSample.root");
	}

	// clean the histos
	delete cosTheta_withCuts;
	cosTheta_withCuts = new TH1D("", "", GLOBAL_BINS, -GLOBAL_absRange, GLOBAL_absRange);
	delete cosTheta_noCuts;
        cosTheta_noCuts = new TH1D("", "", GLOBAL_BINS, -GLOBAL_absRange, GLOBAL_absRange);
        string treeToRead = "testTree_allPairs";
        if (sampleSelector==1) { // with cuts
                treeToRead = "testTree";
                anlz.setCuts(invMassCut, pTransverseCut, EtaCut);
        }
        else if (sampleSelector==2) { // no cuts
                treeToRead = "testTree_allPairs";
                anlz.setCuts(no_invMassCut, no_pTransverseCut, no_EtaCut);
        }
        Nevents = anlz.loopOverEvents(filePath,treeToRead.c_str(), 0,0);
        cout << "Nevents = " << Nevents << endl;
        Nleptons = anlz.getAllLeptons(allgoodLeptons);
        Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
        cout << "Nleptons = " << Nleptons << endl;
        if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
        cout << "\n" << endl;



	// rebin small samples histograms	
	if (sampleSizeSelector==2) { GLOBAL_BINS = 12; }
	if (sampleSizeSelector==3) {
		switch(iModel) {
			case 1: GLOBAL_BINS = 10; break;
			case 2: GLOBAL_BINS = 8; break;
			case 3: GLOBAL_BINS = 8; break;
			default: break;
		}

	}
	TH1D* cosTheta_allRange = new TH1D("", "", GLOBAL_BINS, cosThetamin, cosThetamax);

        // run over the good pairs (passed cuts) and fill the histograms
        for (int k=0 ; k<Nleptons ; k++) {
                invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
                cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
                if (quarkDirectionOrRapidity==2) {
                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
                }
                else if (quarkDirectionOrRapidity==1) {
                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k], true);
                }
                //cosTh = cosThetaBoost;
                cosTh = cosThetaCollinsSoper;
                if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			if (sampleSelector==1) { // with cuts
				if (abs(cosTh)<=GLOBAL_absRange) {
					GLOBAL_cosTheta->push_back(cosTh);
					cosTheta_withCuts->Fill(cosTh);
					//cosTheta_dummy->Fill(cosTh);
				}
			}
			else if (sampleSelector==2) { // no cuts
				if (abs(cosTh)<=GLOBAL_absRange) {
					GLOBAL_cosTheta->push_back(cosTh);
					cosTheta_noCuts->Fill(cosTh);
				}
			}
			cosTheta_allRange->Fill(cosTh);
                }
        }
	// clear the pairs vectors
        anlz.clearAllLeptons();
        if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
        if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
	
	




// to check how many entries in the clean (no cuts) sample.
/*
	treeToRead = "testTree_allPairs";
	anlz.setCuts(no_invMassCut, no_pTransverseCut, no_EtaCut);
        Nevents = anlz.loopOverEvents(filePath,treeToRead.c_str());
        cout << "Nevents = " << Nevents << endl;
        Nleptons = anlz.getAllLeptons(allgoodLeptons);
        Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
        cout << "Nleptons = " << Nleptons << endl;
        if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
        cout << "\n" << endl;


	nEvents_noCuts_allRange = 0;
	Int_t nEvents_noCuts = 0;
        // run over the good pairs (passed cuts) and fill the histograms
        for (int k=0 ; k<Nleptons ; k++) {
                invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
                cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
                if (quarkDirectionOrRapidity==2) {
                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
                }
                else if (quarkDirectionOrRapidity==1) {
                        cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k], true);
                }
                //cosTh = cosThetaBoost;
                cosTh = cosThetaCollinsSoper;
                if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
			if (abs(cosTh)<=GLOBAL_absRange) {
				nEvents_noCuts ++;
			}
			nEvents_noCuts_allRange ++;
                }
        }
        // clear the pairs vectors
        anlz.clearAllLeptons();
        if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
        if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
	cout << "the number of entries of the CLEAN input sample" << endl;
        cout << "nEvents_noCuts = " << nEvents_noCuts << endl;
        cout << "nEvents_noCuts_allRange = " << nEvents_noCuts_allRange << "\n" << endl;
*/






	if (sampleSelector==1) { // with cuts
		GLOBAL_numberOfEvents = (int)cosTheta_withCuts->GetEntries();
	}
	else if (sampleSelector==2) { // no cuts
		GLOBAL_numberOfEvents = (int)cosTheta_noCuts->GetEntries();
	}
//	if (takeItherEfficiency) { GLOBAL_numberOfEvents = (int)cosTheta_withCuts->GetEntries(); }

	cout << "the number of entries of the input sample in the range +-" << strFitRange << endl;
	cout << "cosTheta_withCuts = " << cosTheta_withCuts->GetEntries() << endl;
	cout << "cosTheta_noCuts = " << cosTheta_noCuts->GetEntries() << "\n" << endl;
	cout << "cosTheta_allRange = " << cosTheta_allRange->GetEntries() << "\n" << endl;


	// Initialize minuit, set initial values etc. of parameters.
	int npar = 2;      // the number of parameters
	if (sSampleSize != "_LrefinvFb") {
		npar = 1; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	}
        TMinuit minuit(npar);
        minuit.SetFCN(fcn);

        double par[npar];        // the start values
        double stepSize[npar];   // step sizes
        double minVal[npar];     // minimum bound on parameter
        double maxVal[npar];     // maximum bound on parameter
        string parName[npar];

	if (sSampleSize == "_LrefinvFb") {
	        par[0]      = 0.0;       // a guess
	        stepSize[0] = 1e-6;      // take e.g. 0.1 of start value
	        minVal[0]   = -10.;      // if min and max values = 0, parameter is unbounded.
	        maxVal[0]   = +10.;
	        parName[0]  = "A0";
	
	        par[1]      = 0.5;       // a guess
	        stepSize[1] = 1e-6;      // take e.g. 0.1 of start value
	        minVal[1]   = -10.;      // if min and max values = 0, parameter is unbounded.
	        maxVal[1]   = +10.;
	        parName[1]  = "A4";
	}
	else {
		par[0]      = 0.5;  //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		stepSize[0] = 1e-6; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		minVal[0]   = -10.; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		maxVal[0]   = +10.; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		parName[0]  = "A4"; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	}


        for (int i=0; i<npar; i++) {
                minuit.DefineParameter(i, parName[i].c_str(), par[i], stepSize[i], minVal[i], maxVal[i]);
        }


        // Do the minimization!
        minuit.Migrad();         // Minuit's best minimization algorithm
        double outpar[npar], err[npar];
        for (int i=0; i<npar; i++){
                minuit.GetParameter(i,outpar[i],err[i]);
        }


        // Plot the result.  For this example plot x values as tick marks.
        //TCanvas *canv = new TCanvas("canv","Minuit",0,0,1400,800);

        double xmin = -GLOBAL_absRange;
        double xmax =  GLOBAL_absRange;
	double dxAllRange   = 2.*cosThetamax/(double)GLOBAL_BINS;
	double dxSmallRange = 2.*GLOBAL_absRange/(double)GLOBAL_BINS;
	//double scale = (cosTheta_allRange->GetEntries()*dxAllRange) / (GLOBAL_numberOfEvents * dxSmallRange);
	double scale = dxAllRange / dxSmallRange;
	GLOBAL_scale = scale;
        TF1* func = NULL;
	if (abs(GLOBAL_absRange)==1.) { func = new TF1("funcplot", unBiasedCollinsSoperPdf, xmin, xmax, npar); }
	else if (abs(GLOBAL_absRange)<1.) { func = new TF1("funcplot", scaledFunction, xmin, xmax, npar); }
        func->SetParameters(outpar);


//	if      (sampleSelector==1) { cosTheta_withCuts->Draw("sames"); }
//      else if (sampleSelector==2) { cosTheta_noCuts->Draw("sames"); }

        func->SetLineStyle(1);             //  1 = solid, 2 = dashed, 3 = dotted
        func->SetLineColor(1);             //  black (default)
        func->SetLineWidth(1);

        func->GetXaxis()->SetTitle("cos#left(#theta '#right)");
        func->GetYaxis()->SetTitle("pdf");

	double A0  = 0., A4  = 0.;
        double dA0 = 0., dA4 = 0.;

	if (sSampleSize == "_LrefinvFb") {
	        minuit.GetParameter(0,A0,dA0);
		minuit.GetParameter(1,A4,dA4);
	}
	else {
		minuit.GetParameter(0,A4,dA4);
	}
        cout << "A0 = " << A0 << "+/-" << dA0 << endl;
        cout << "Afb = " << A4*3./8. << "+/-" << dA4*3./8. << endl;
	cout << "\n" << endl;


	// calculate chi2
	Double_t chi2=0.;
	TAxis *xaxis = cosTheta_allRange->GetXaxis();
	Double_t hVal;
	Double_t binCenter;
	Double_t fVal;
	Double_t fNorm = (Double_t)( 2.*GLOBAL_absRange*(1.+0.5*A0) + (1.-1.5*A0)*(2.*GLOBAL_absRange*GLOBAL_absRange*GLOBAL_absRange/3.) );
	Double_t hIntegral = GLOBAL_numberOfEvents * 2./(Double_t)GLOBAL_BINS;
	Int_t DOF = 0;


//-----------------------------------------------------------------------------------------
//	double dx = 2.*GLOBAL_absRange/(double)GLOBAL_BINS;
//	int    binLocationIndex = 0;
//	if (x!=GLOBAL_absRange) { binLocationIndex = (int)((x+GLOBAL_absRange)/dx); }
//	else                    { binLocationIndex = GLOBAL_BINS-1; }
//-----------------------------------------------------------------------------------------


	for (Int_t i=0 ; i<GLOBAL_BINS ; ++i) {
		binCenter = xaxis->GetBinCenter(i);
		if (abs(binCenter) <= GLOBAL_absRange) {
			//double* x(&binCenter);
			//fVal = (Double_t)scaledFunction(x,par);
			fVal = (Double_t)(1.+0.5*A0 + A4*binCenter + (1.-1.5*A0)*binCenter*binCenter) / fNorm * hIntegral;
			hVal = cosTheta_allRange->GetBinContent(i);
			cout << "binCenter = " << binCenter << "\t" << "fVal = " << fVal << "\t" << "hVal = " << hVal << endl;
			if (hVal!=0) {
				chi2 += (fVal-hVal)*(fVal-hVal)/hVal;
			}
			else {
				chi2 += (fVal-hVal)*(fVal-hVal)/1.;
			}
			DOF ++;
			//delete x;
		}
	}
	chi2 /= DOF;
	cout << "DOF/BINS = " << DOF << "/" << GLOBAL_BINS << endl;
	cout << "chi2 = " << chi2 << endl;
	cout << "\n" << endl;


	// ------ legends -----------------------------------------------------------------------------------------------------
	stringstream sstrm;
	string str;
	TLegend *leg = new TLegend(0.35,0.7,0.65,0.9);
	sstrm << GLOBAL_absRange;
	sstrm >> str;
	string label1 = "Generated data #left(" + sModel + "#right)";
	string label2 = "ML fit #left(" + sModel + "#right),  in #left|cos#theta '#right|#leq" + str;
	if (iModel ==3) {
		label1 = "Generated data #left(Z'#right)";
		label2 = "ML fit #left(Z'#right),  in #left|cos#theta '#right|<" + str;
	}
	leg->AddEntry(cosTheta_allRange,label1.c_str(),"P");
	leg->AddEntry(func,label2.c_str(),"L");
//	if (sampleSelector==1) {
//		leg->AddEntry(cosTheta_withCuts,label1.c_str(),"P");
//		leg->AddEntry(func,label2.c_str(),"L");
//	}
//	else if (sampleSelector==2) {
//		leg->AddEntry(cosTheta_noCuts,label1.c_str(),"P");
//		leg->AddEntry(func,label2.c_str(),"L");
//	}


	// frames
	double maxHeight = 0;
	//if      (sampleSelector==1) { maxHeight = cosTheta_withCuts->GetMaximum() + sqrt(cosTheta_withCuts->GetMaximum()); }
	//else if (sampleSelector==2) { maxHeight = cosTheta_noCuts->GetMaximum() + sqrt(cosTheta_noCuts->GetMaximum()); }
	maxHeight = cosTheta_allRange->GetMaximum() + sqrt(cosTheta_allRange->GetMaximum());
	TH2 *frameHisto = new TH2D("", "", 1,cosThetamin,cosThetamax, 1,0.,1.05*maxHeight);
	frameHisto->SetXTitle(xtitle.c_str());
	frameHisto->SetYTitle(ytitle.c_str());


	cout << "\n---finalizing---\n" << endl;

	// draw the histos
	frameHisto->Draw();
//	if      (sampleSelector==1) { cosTheta_withCuts->Draw("E1,sames"); }
//	else if (sampleSelector==2) { cosTheta_noCuts->Draw("E1,sames"); }
	cosTheta_allRange->Draw("E1,sames");
	func->Draw("sames");
	leg->Draw("sames");

	
	// finalize the graphics
	canv->Update();
	canv->Print(TMinuit_collinssoperRootFileName.c_str(),"root");
	canv->Update();
	canv->Print(TMinuit_collinssoperEpsFileName.c_str());
	TMinuit_collinssoperHistFile->Write();

	// wait so the plots can hold on
	canv->WaitPrimitive();

	// Done.
	return 0;
}

