/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------
//#include "analyzer.h"

using namespace std;
using namespace Moses1;
using namespace Pythia8;
//using namespace RooFit;

int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;

	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	gStyle->SetOptStat(11);

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();

	// set the prefix
	string dirName = "/KKvsZE6vsSMstudy/";
	string outFileName = dirName + "acceptance";
	cout << "\n" << endl;


	// declare the relevant classes
	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)
	dileptonAnalyzer anlz;
	Particle particle;

	// set cuts
	double invMassCut=1000.;
	double pTransverseCut=10.;
	double EtaCut=2.5;
	anlz.setCuts(invMassCut, /*pTransverseCut=*/0., /*EtaCut=*/1e10);


	// choose reference frame
	int referenceFrameSelector = 0;
	cout << "choose reference frame to calculate cos(theta)" << endl;
	cout << "(1) Helicity frame or (2) Collins-Soper frame" << endl;
	cout << "..." << endl;
	cin >> referenceFrameSelector;
	if (referenceFrameSelector != 1 && referenceFrameSelector != 2) {
		cout << "invalid option, setting to default = 1" << endl;
		referenceFrameSelector = 1;
	}
	string cosThetaTitle;
	if (referenceFrameSelector == 1) {cosThetaTitle = "Helicity";}
	else if (referenceFrameSelector == 2) {cosThetaTitle = "Collins-Soper";}

	
	int sampleSizeSelector = 1;
	cout << "select input sample size: 1M(1), L=500fb^-1(2), L=100fb^-1(3)..." << endl;
	cin >> sampleSizeSelector;
	if (sampleSizeSelector<1 || sampleSizeSelector>3) {
		cout << "invalid option, set to 1 (default)" << endl;
		sampleSizeSelector = 1;
	}
	string sSampleSize = "";
	switch (sampleSizeSelector) {
		case 1: sSampleSize = "_1MeventsSample"; break;
		case 2: sSampleSize = "_L500invFbSample"; break;
		case 3: sSampleSize = "_L100invFbSample"; break;
		default: break;
	}

	outFileName += sSampleSize; // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	double cosThMax = 1.;
	cout << "enter a limit for |cos(theta)|...." << endl;
	cin >> cosThMax;
	if (cosThMax > 1. || cosThMax < 0.) {
		cout << "invalid choice: |cos(theta)|=" << cosThMax << "  setting it to |cos(theta)|=1" << endl;
		cosThMax = 1.;
	}



	// declare kinematics histograms and root items
	string acceptanceRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_mass" + ".root");
	string acceptanceEpsFileName  = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_mass" + ".eps");
	
	string angularAcceptanceRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_angular" + ".root");
	string angularAcceptanceEpsFileName  = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_angular" + ".eps");

	string rapidityAcceptanceRootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_rapidity" + ".root");
	string rapidityAcceptanceEpsFileName  = commonOps.checkANDsetFilepath("MOSESDATA", outFileName + "_rapidity" + ".eps");


	TFile *acceptanceHistFile = new TFile(acceptanceRootFileName.c_str(),"recreate");
	TCanvas *canv = new TCanvas("canv","Kaluza-Klein",0,0,1400,800);
//	canv->Divide(1,2);
	TFile *angularAcceptanceHistFile = new TFile(angularAcceptanceRootFileName.c_str(),"recreate");
	TCanvas *canvAngular = new TCanvas("canvAngular","Kaluza-Klein",0,0,1400,800);

	TFile *rapidityAcceptanceHistFile = new TFile(rapidityAcceptanceRootFileName.c_str(),"recreate");
	TCanvas *canvRapidity = new TCanvas("canvRapidity","Kaluza-Klein",0,0,1400,800);
	//canvRapidity->Divide(2,2);	


	// boundries and bins of histograms
	Int_t invMnBins = 50;
	Int_t cosThetanBins = 22;
        Double_t invMmin = 1000.,    invMmax = 6000.;
	Double_t cosThetamin = -1., cosThetamax = 1.;
	Int_t    etanBins = 48; // 80
	Double_t etamin = -3., etamax = 3.;


	// choose mHat & cosTheta binnings
	switch(sampleSizeSelector) {
		case 1:	invMnBins = 50; cosThetanBins = 22; etanBins = 48; break; // 1M sample
		case 2: invMnBins = 14; cosThetanBins = 10; etanBins = 12; break; // 500 fb^-1 sample
		case 3: invMnBins = 14; cosThetanBins = 8;  etanBins = 12; break; // 100 fb^-1 sample
		default: break;
	}
	cout << "invMnBins     = " << invMnBins << endl;
	cout << "cosThetanBins = " << cosThetanBins << endl;

	invMnBins = 40; // 50
	invMmin = 1000.;
	invMmax = 5500.; // 5500

	cosThetanBins = 200; //30 for zooming;
	cosThetamin   = -1.; //-1.1;
	cosThetamax   =  1.; //1.1;

	etanBins = 60;
	etamin = -3.;
	etamax = 3.;



	// titles for the cos(theta) plots
//	string title  = "SM vs KK vs Z': cosTheta (in " + cosThetaTitle + " frame) *Normalized*";
	string title  = "";
	string xtitle = "cos#theta'";


	// declaration of histograms
	//----- SM -----------------------------------------------------------------------------------------------------
	TH1 *SM_invMass = new TH1D("", "", invMnBins, invMmin, invMmax);
	SM_invMass->SetLineStyle(5);
	SM_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	SM_invMass->SetYTitle("Trigger Acceptance");
	TH1 *SM_invMass_noCuts = new TH1D("", "", invMnBins, invMmin, invMmax);

	TH1 *SM_cosTheta = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	SM_cosTheta->SetLineStyle(5);
	SM_cosTheta->SetXTitle(xtitle.c_str());
	SM_cosTheta->SetYTitle("Trigger Acceptance");
	TH1 *SM_cosTheta_noCuts = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);

	TH1 *SM_eta = new TH1D("", "", etanBins, etamin, etamax);
	SM_eta->SetLineStyle(5);
	SM_eta->SetXTitle("y_{#mu^{+}#mu^{-}}");
	SM_eta->SetYTitle("Trigger Acceptance");
	TH1 *SM_eta_noCuts = new TH1D("", "", etanBins, etamin, etamax);
	//TH1 *SM_eta_mu = new TH1D("eta_mu", "eta_mu", etanBins, etamin, etamax);
	//TH1 *SM_eta_mubar = new TH1D("eta_mubar", "eta_mubar", etanBins, etamin, etamax);
	
	//----- KK -----------------------------------------------------------------------------------------------------
	TH1 *KK_invMass = new TH1D("", "", invMnBins, invMmin, invMmax);
	KK_invMass->SetLineStyle(1);
	KK_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	KK_invMass->SetYTitle("Trigger Acceptance");
	TH1 *KK_invMass_noCuts = new TH1D("", "", invMnBins, invMmin, invMmax);

	TH1 *KK_cosTheta = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	KK_cosTheta->SetLineStyle(1);
	KK_cosTheta->SetXTitle(xtitle.c_str());
	KK_cosTheta->SetYTitle("Trigger Acceptance");
	TH1 *KK_cosTheta_noCuts = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);

	TH1 *KK_eta = new TH1D("", "", etanBins, etamin, etamax);
	KK_eta->SetLineStyle(1);
	KK_eta->SetXTitle("y_{#mu^{+}#mu^{-}}");
	KK_eta->SetYTitle("Trigger Acceptance");
	TH1 *KK_eta_noCuts = new TH1D("", "", etanBins, etamin, etamax);
	//TH1 *KK_eta_mu = new TH1D("eta_mu", "eta_mu", etanBins, etamin, etamax);
        //TH1 *KK_eta_mubar = new TH1D("eta_mubar", "eta_mubar", etanBins, etamin, etamax);
	
	//----- Z' -----------------------------------------------------------------------------------------------------
	TH1 *Zprime_invMass = new TH1D("", "", invMnBins, invMmin, invMmax);
	Zprime_invMass->SetLineStyle(3);
	Zprime_invMass->SetXTitle("#sqrt{#hat s}(#mu^{-}#mu^{+})[GeV]");
	Zprime_invMass->SetYTitle("Trigger Acceptance");
	TH1 *Zprime_invMass_noCuts = new TH1D("", "", invMnBins, invMmin, invMmax);

	TH1 *Zprime_cosTheta = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);
	Zprime_cosTheta->SetLineStyle(3);
	Zprime_cosTheta->SetXTitle(xtitle.c_str());
	Zprime_cosTheta->SetYTitle("Trigger Acceptance");
	TH1 *Zprime_cosTheta_noCuts = new TH1D("", "", cosThetanBins, cosThetamin, cosThetamax);

	TH1 *Zprime_eta = new TH1D("", "", etanBins, etamin, etamax);
        Zprime_eta->SetLineStyle(3);
        Zprime_eta->SetXTitle("y_{#mu^{+}#mu^{-}}");
        Zprime_eta->SetYTitle("Trigger Acceptance");
        TH1 *Zprime_eta_noCuts = new TH1D("", "", etanBins, etamin, etamax);
	//TH1 *Zprime_eta_mu = new TH1D("eta_mu", "eta_mu", etanBins, etamin, etamax);
        //TH1 *Zprime_eta_mubar = new TH1D("eta_mubar", "eta_mubar", etanBins, etamin, etamax);

	// ------ legends -----------------------------------------------------------------------------------------------------
	TLegend *leg1 = new TLegend(0.6,0.4,0.86,0.6);
	leg1->AddEntry(SM_invMass,"SM","L");
	leg1->AddEntry(KK_invMass,"KK","L");
	leg1->AddEntry(Zprime_invMass,"Z'_{SM}","L");

	TLegend *leg2 = new TLegend(0.6,0.4,0.86,0.6);
	leg2->AddEntry(SM_cosTheta,"SM","L");
	leg2->AddEntry(KK_cosTheta,"KK","L");
	leg2->AddEntry(Zprime_cosTheta,"Z'_{SM}","L");

	TLegend *leg3 = new TLegend(0.7,0.6,0.96,0.8);
        leg3->AddEntry(SM_eta,"SM","L");
        leg3->AddEntry(KK_eta,"KK","L");
        leg3->AddEntry(Zprime_eta,"Z'_{SM}","L");
	
	//----------------------------------------------------------------------------------------------------------
	
	string filePath;
	int Nevents;
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons, Naleptons;
	double cosThetaBoost, cosThetaCollinsSoper;
	double invariantMass, pTLepton, pTALepton, cosTh=0, etaLepton, etaALepton, yQ;

	// to check the asymmetry in a narrow range arrownd the peak
	double mZ_known = physPrm.mStar;
	double mZ_lowLimit = physPrm.mStar-2000.;
	double mZ_highLimit = physPrm.mStar+1000.;
	cout << "\n" << "mZ_known=" << mZ_known << "  mZ_lowLimit=" << mZ_lowLimit << "  mZ_highLimit=" << mZ_highLimit << "\n" << endl;
	
	string treeType = "testTree_allPairs";

	int sm_badEvents = 0, kk_badEvents = 0, e6_badEvents = 0;

	// loop over 2 files (with / without Initial State Radiation)
	for (int f=1 ; f<=3 ; f++) {
		//-------------------------------------------------------------------------------------------------------------------
		// read the trees and put all good lepton pairs into 2 vectors
		//----- 1M
		if (sampleSizeSelector == 1) { //1M smaples
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"SM_extractedParticles_2trees_1Msample.root");
			}
			else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"KK_extractedParticles_2trees_1Msample.root");
			}
			else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"1Msamples/"+"E6_extractedParticles_2trees_1Msample.root");
			}
		}

		//----- 500
		else if (sampleSizeSelector == 2) { // 500[fb^-1]
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+"SM_extractedParticles_2trees_500inversefemtobarnSample.root");
			}
                        else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+"KK_extractedParticles_2trees_500inversefemtobarnSample.root");
                        }
                        else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"500inversefemtobarn/"+"E6_extractedParticles_2trees_500inversefemtobarnSample.root");
			}
		}

		//----- 100
		else if (sampleSizeSelector == 3) { // 100[fb^-1]
			if (f==1) { // SM
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100inversefemtobarn/"+"SM_extractedParticles_2trees_100inversefemtobarnSample.root");
			}
			else if (f==2) { // KK
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100inversefemtobarn/"+"KK_extractedParticles_2trees_100inversefemtobarnSample.root");
			}
			else if (f==3) { // E6
				filePath = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"100inversefemtobarn/"+"E6_extractedParticles_2trees_100inversefemtobarnSample.root");
			}
		}

		Nevents = anlz.loopOverEvents(filePath, treeType.c_str(), 0,0);
		cout << "Nevents = " << Nevents << endl;
		// get all the good pairs (after cuts)
		Nleptons = anlz.getAllLeptons(allgoodLeptons);
		Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
		cout << "Nleptons = " << Nleptons << endl;
		if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}
		cout << "\n" << endl;
		//-------------------------------------------------------------------------------------------------------------------
		// run over the good pairs and fill the histograms
		for (int k=0 ; k<Nleptons ; k++) {
			invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
			yQ = anlz.y(allgoodLeptons[k], allgoodaLeptons[k]);
			pTLepton = allgoodLeptons[k].pT;
			pTALepton = allgoodaLeptons[k].pT;
			etaLepton = allgoodLeptons[k].eta;
			etaALepton = allgoodaLeptons[k].eta;
			cosThetaBoost = anlz.cosThetaBoost(allgoodLeptons[k], allgoodaLeptons[k]);
			cosThetaCollinsSoper = anlz.cosThetaCollinsSoper(allgoodLeptons[k], allgoodaLeptons[k]);
			if (referenceFrameSelector == 1) {cosTh = cosThetaBoost;}
			else if (referenceFrameSelector == 2) {cosTh = cosThetaCollinsSoper;}
			// fill the histos
			if (f==1) {
				if (invariantMass>=invMassCut) {
					if (pTLepton>=pTransverseCut && pTALepton>=pTransverseCut && abs(etaLepton)<EtaCut && abs(etaALepton)<EtaCut) {
						SM_invMass->Fill( (Double_t)invariantMass );
						SM_eta->Fill( (Double_t)yQ );
						if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
							SM_cosTheta->Fill( (Double_t)cosTh );
						}
					}
					else { sm_badEvents ++; }
					SM_invMass_noCuts->Fill( (Double_t)invariantMass );
					SM_eta_noCuts->Fill( (Double_t)yQ );
					//SM_eta_mu->Fill( (Double_t)etaLepton);
					//SM_eta_mubar->Fill( (Double_t)etaALepton);
					if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
						SM_cosTheta_noCuts->Fill( (Double_t)cosTh );
					}
				}
			}
			else if (f==2) {
				if (invariantMass>=invMassCut) {
					if (pTLepton>=pTransverseCut && pTALepton>=pTransverseCut && abs(etaLepton)<EtaCut && abs(etaALepton)<EtaCut) {
						KK_invMass->Fill( (Double_t)invariantMass );
						KK_eta->Fill( (Double_t)yQ );
						if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
							KK_cosTheta->Fill( (Double_t)cosTh );
						}
					}
					else { kk_badEvents ++; }
					KK_invMass_noCuts->Fill( (Double_t)invariantMass );
					KK_eta_noCuts->Fill( (Double_t)yQ );
					//KK_eta_mu->Fill( (Double_t)etaLepton);
					//KK_eta_mubar->Fill( (Double_t)etaALepton);
					if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
						KK_cosTheta_noCuts->Fill( (Double_t)cosTh );
					}
				}
			}
			else if (f==3) {
				if (invariantMass>=invMassCut) {
					if (pTLepton>=pTransverseCut && pTALepton>=pTransverseCut && abs(etaLepton)<EtaCut && abs(etaALepton)<EtaCut) {
						Zprime_invMass->Fill( (Double_t)invariantMass );
						Zprime_eta->Fill( (Double_t)yQ );
						if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
							Zprime_cosTheta->Fill( (Double_t)cosTh );
						}
					}
					else { e6_badEvents ++; }
					Zprime_invMass_noCuts->Fill( (Double_t)invariantMass );
					Zprime_eta_noCuts->Fill( (Double_t)yQ );
					//Zprime_eta_mu->Fill( (Double_t)etaLepton);
					//Zprime_eta_mubar->Fill( (Double_t)etaALepton);
					if (invariantMass >= mZ_lowLimit && invariantMass <= mZ_highLimit) {
						Zprime_cosTheta_noCuts->Fill( (Double_t)cosTh );
					}
				}
			}
		}
		// clear the pairs vectors
		anlz.clearAllLeptons();
		if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
		if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }
	}
	//---------------------------------------------------------------------------------------------------------

	cout << "sm_badEvents = " << sm_badEvents << endl;
	cout << "kk_badEvents = " << kk_badEvents << endl;
	cout << "e6_badEvents = " << e6_badEvents << endl; 


	// calculate the acceptances:
	SM_invMass->Divide(SM_invMass_noCuts);
	Zprime_invMass->Divide(Zprime_invMass_noCuts);
	KK_invMass->Divide(KK_invMass_noCuts);

	SM_cosTheta->Divide(SM_cosTheta_noCuts);
	Zprime_cosTheta->Divide(Zprime_cosTheta_noCuts);
	KK_cosTheta->Divide(KK_cosTheta_noCuts);

	SM_eta->Divide(SM_eta_noCuts);
        Zprime_eta->Divide(Zprime_eta_noCuts);
        KK_eta->Divide(KK_eta_noCuts);

	// draw the histos
	canv->cd();
	SM_invMass->SetMaximum(1.05);
	SM_invMass->SetMinimum(0.);
	SM_invMass->Draw();
	//SM_invMass->DrawCopy("e1sames");
	Zprime_invMass->Draw("sames");
	KK_invMass->Draw("sames");
	//KK_invMass->DrawCopy("e1sames");
	leg1->Draw("sames");
	canv->Update();

	canvAngular->cd();
	SM_cosTheta->SetMaximum(1.05);
	SM_cosTheta->SetMinimum(0.); // 0.8 for zooming
	SM_cosTheta->Draw();
	//SM_cosTheta->DrawCopy("e1sames");
	Zprime_cosTheta->Draw("sames");
	KK_cosTheta->Draw("sames");
	//KK_cosTheta->DrawCopy("e1sames");
	leg2->Draw("sames");
	canvAngular->Update();

	canvRapidity->cd();
	//canvRapidity->cd(1);
        SM_eta->SetMaximum(1.05);
	//SM_eta->SetMaximum(1.05*KK_eta->GetMaximum());
	SM_eta->SetMinimum(0.);
        SM_eta->Draw();
        //SM_cosTheta->DrawCopy("e1sames");
        Zprime_eta->Draw("sames");
        KK_eta->Draw("sames");
        //KK_cosTheta->DrawCopy("e1sames");
        leg3->Draw("sames");
	canvRapidity->Update();

/*
	canvRapidity->cd(2);
	SM_eta_noCuts->SetMaximum(1.05*KK_eta_noCuts->GetMaximum());
	SM_eta_noCuts->Draw();
	Zprime_eta_noCuts->Draw("sames");
	KK_eta_noCuts->Draw("sames");
	canvRapidity->cd(3);
	SM_eta_mu->SetMaximum(1.05*KK_eta_mu->GetMaximum());
        SM_eta_mu->Draw();
        Zprime_eta_mu->Draw("sames");
        KK_eta_mu->Draw("sames");
	canvRapidity->cd(4);
	SM_eta_mubar->SetMaximum(1.05*KK_eta_mubar->GetMaximum());
        SM_eta_mubar->Draw();
        Zprime_eta_mubar->Draw("sames");
        KK_eta_mubar->Draw("sames");
	canvRapidity->Update();
*/
	


	cout << "\n---finalizing---\n" << endl;
	
	// finalize the graphics
	canv->Print(acceptanceRootFileName.c_str(),"root");
	canv->Update();
	canv->Print(acceptanceEpsFileName.c_str());
	acceptanceHistFile->Write();

	canvAngular->Print(angularAcceptanceRootFileName.c_str(),"root");
	canvAngular->Update();
	canvAngular->Print(angularAcceptanceEpsFileName.c_str());
	angularAcceptanceHistFile->Write();

	canvRapidity->Print(rapidityAcceptanceRootFileName.c_str(),"root");
        canvRapidity->Update();
        canvRapidity->Print(rapidityAcceptanceEpsFileName.c_str());
        rapidityAcceptanceHistFile->Write();


	// wait so the plots can hold on
	canv->WaitPrimitive();
	canvRapidity->WaitPrimitive();
	canvAngular->WaitPrimitive();

	// Done.
	return 0;
}

