/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------
//#include "analyzer.h"

using namespace std;
using namespace Moses1;
using namespace Pythia8;

int main() {
	cout << "Holy Moses..." << endl;
	
	// set styles
	gStyle->SetOptStat(11);
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	//gStyle->SetErrorX(0.5);

	// declare some histos and related root entities	
	commonOperations commonOps;
	string outFileName = "/OVERLOOKstudy/KK_overlook";
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".eps");

	TFile *histFile = new TFile(rootFileName.c_str(),"recreate");
	TCanvas *c1 = new TCanvas("c1","Kaluza-Klein",0,0,1400,800);
	c1->Divide(2,2);

	// plots of analysis on muon pairs
	TVirtualPad *pad1=c1->cd(1);
	pad1->SetLogy();
	
	//TVirtualPad *pad2=c1->cd(2);

	TVirtualPad *pad3=c1->cd(3);
	pad3->SetLogy();

	//TVirtualPad *pad4=c1->cd(4);

	// boundries and bins of histograms
	Int_t    invMnBins = 50;
	Double_t invMmin   = 1000.;
	Double_t invMmax   = 6000.;

	Int_t    pTnBins = 50;
	Double_t pTmin   = 10.;
	Double_t pTmax   = 2000.;

	Int_t    etanBins = 48;
	Double_t etamin   = -4.;
	Double_t etamax   = 4.;
	
	Int_t    cosThetanBins = 40;
	Double_t cosThetamin   = -2.;
	Double_t cosThetamax   = 2.;

	// declaration of histograms
	TH1 *invMass = new TH1D("m(mu-,mu+)", "muon pairs invariant mass", invMnBins, invMmin, invMmax);
	invMass->SetLineColor(kBlack);
	invMass->SetXTitle("invariantMass(mu-,mu+)[GeV]");
	invMass->SetYTitle("Events");
	invMass->SetTitle("Muon pairs invariant mass distribution");
	//invMass->SetMarkerStyle(20);

	TH1 *pT = new TH1D("pT(mu-,mu+)", "muon pairs pT", pTnBins, pTmin, pTmax);
	pT->SetLineColor(kBlack);
	pT->SetXTitle("pT(mu-,mu+)[GeV]");
	pT->SetYTitle("Events");
	pT->SetTitle("Muon pairs pT distribution");
	//pT->SetMarkerStyle(20);

	TH1 *eta = new TH1D("eta(mu-)", "muons eta", etanBins, etamin, etamax);
	eta->SetLineColor(kBlack);
	eta->SetXTitle("eta(mu-)");
	eta->SetYTitle("Events");
	eta->SetTitle("Muons(mu-) Eta distribution");
	//eta->SetMarkerStyle(20);

	TH1 *cosTheta = new TH1D("cosTheta(mu-)", "muons cosTheta", cosThetanBins, cosThetamin, cosThetamax);
	cosTheta->SetLineColor(kBlack);
	cosTheta->SetXTitle("cosTheta(mu-) c.m.f");
	cosTheta->SetYTitle("Events");
	cosTheta->SetTitle("Muons(mu-) cosTheta (in c.m.f) distribution");
	//cosTheta->SetMarkerStyle(20);

	//----------------------------------------------------------------------------------------------------------
	// declare the relevant classes
	dileptonAnalyzer anlz;
	Particle particle;
	RotBstMatrix bst;
	
	// set cuts
	double invMassCut=1000.;
	double pTransverseCut=10.;
	double EtaCut=2.5;
	anlz.setCuts(invMassCut, pTransverseCut, EtaCut);
	
	// start loop over events file(/files)
	//string filePath = commonOps.checkANDsetFilepath("MOSESDATA", "/OVERLOOKstudy/KK_extractedParticles.dat");
	string filePath = commonOps.checkANDsetFilepath("MOSESDATA", "/OVERLOOKstudy/KK_extractedParticles_10GeVpTcut.dat");
	int Nevents = anlz.loopOverEvents(filePath);
	cout << "Nevents = " << Nevents << endl;
	
	// get all the good pairs (after cuts)
	vector<fullRecord> allgoodLeptons, allgoodaLeptons;
	int Nleptons = anlz.getAllLeptons(allgoodLeptons);
	int Naleptons = anlz.getAllaLeptons(allgoodaLeptons);
	cout << "Nleptons = " << Nleptons << endl;
	if (Nleptons != Naleptons) {cout << "warning: Nleptons != Naleptons, exitting" << endl;    exit(-1);}

	// run over the good pairs and fill the histograms
	double invariantMass, pTransverse, Rapidity, cosTh;
	Vec4 p1,p2,p1old,p2old,pSum;
	for (int k=0 ; k<Nleptons ; k++) {
		invariantMass = anlz.invMass(allgoodLeptons[k], allgoodaLeptons[k]);
		pTransverse = anlz.pT(allgoodLeptons[k], allgoodaLeptons[k]);
		Rapidity = anlz.y(allgoodLeptons[k], allgoodaLeptons[k]);
		p1.px(allgoodLeptons[k].px);
		p1.py(allgoodLeptons[k].py);
		p1.pz(allgoodLeptons[k].pz);
		p1.e(allgoodLeptons[k].e);
		p2.px(allgoodaLeptons[k].px);
                p2.py(allgoodaLeptons[k].py);
                p2.pz(allgoodaLeptons[k].pz);
                p2.e(allgoodaLeptons[k].e);
		pSum = p1 + p2;
		//cout << "before pSum=" << pSum << endl;
		p1.bstback(pSum);
		p2.bstback(pSum);
		//bst.toCMframe(p1,p2); // boost and rotate to the rest frame of p1 and p2, with p1 along the +z axis.
		pSum = p1 + p2;
		//cout << "after  pSum=" << pSum << endl;
		cosTh = p1.pz()/sqrt(p1.px()*p1.px() + p1.py()*p1.py() + p1.pz()*p1.pz()); // the cosine of the opening angle between the vectors.
		//--- begin fill ---//
		invMass->Fill( (Double_t)invariantMass );
		pT->Fill( (Double_t)pTransverse );
		eta->Fill( (Double_t)allgoodLeptons[k].eta );
		//if (Rapidity > 0) { cosTheta->Fill( (Double_t)cos(allgoodLeptons[k].theta) ); }
		//else { cosTheta->Fill( -1.*(Double_t)cos(allgoodLeptons[k].theta) ); }
		if (Rapidity > 0) { cosTheta->Fill( (Double_t)cosTh ); }
		else { cosTheta->Fill( -1.*(Double_t)cosTh ); }
		//---- end fill ----//
	}

	// clear the pairs vectors
	anlz.clearAllLeptons();
	if (allgoodLeptons.size() != 0) { allgoodLeptons.clear(); }
	if (allgoodaLeptons.size() != 0) { allgoodaLeptons.clear(); }	
	//----------------------------------------------------------------------------------------------------------

	// calculate AFB:
	Double_t Nplus = 0;
	Double_t Nminus = 0;
	Double_t cost = cosThetamin;
	Double_t dcost = (cosThetamax - cosThetamin)/(Double_t)cosThetanBins;
	for (Int_t i=0 ; i<cosThetanBins ; i++) {
		if (cost < 0) { Nminus += (Double_t)cosTheta->GetBinContent(i); }
		else if (cost >= 0) { Nplus += (Double_t)cosTheta->GetBinContent(i); }
		cost += dcost;
	}
	cout << "AFB = " << (Nplus-Nminus)/(Nplus+Nminus) << endl;


	// draw the histos
	c1->cd(1);
	invMass->Draw();
	invMass->DrawCopy("esames");
	c1->cd(2);
	eta->SetMaximum(45.);
	eta->Draw();
	eta->DrawCopy("esames");
	c1->cd(3);
	pT->Draw();
	pT->DrawCopy("esames");
	c1->cd(4);
	cosTheta->SetMaximum(50.);
	cosTheta->Draw();
	cosTheta->DrawCopy("esames");
	
	// finalize the graphics
	c1->Print(rootFileName.c_str(),"root");
	c1->Update();
	c1->Print(epsFileName.c_str());
	histFile->Write();

	// Done.
	return 0;
}

