/************************************************************/
/* an example program to demonstrate a basic use of pythia8 */
/* for an LHC setup                                         */
/************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace Moses1;
using namespace Pythia8;

int main() {
	// Generator. Process selection. LHC initialization. Histogram.
	Pythia pythia;

        //------------------------------------------------------
        // Chosen new PDF set; LHAPDF file name conventions.
        //string pdfSet = "cteq61.LHgrid";
        //string pdfSet = "cteq65.LHgrid";
        //string pdfSet = "cteq65.LHpdf";
        //string pdfSet = "cteq5l.LHgrid";
        string pdfSet = "MRST2001lo.LHgrid";
        //string pdfSet = "cteq6.LHpdf";
        //string pdfSet = "MRST2001E.LHgrid";
        int pdfMember_int = 0;
        string pdfMember_str = "0";
        stringstream s;
        if (pdfSet=="cteq6.LHpdf" || pdfSet=="cteq65.LHpdf" || pdfSet=="cteq65.LHgrid") {
                cout << "pdf set is: " << pdfSet << endl;
                cout << "please chose a pdf member (0->40)...";
                cin >> pdfMember_int;
                s << pdfMember_int;
                s >> pdfMember_str;
                if (pdfSet=="cteq6.LHpdf" && pdfMember_int == 0) { pdfSet = "cteq6m.LHpdf"; }
                s.clear();
        }
        else if (pdfSet=="MRST2001E.LHgrid") {
                cout << "pdf set is: " << pdfSet << endl;
                cout << "please chose a pdf member (0=central, 1->30=errors)...";
                cin >> pdfMember_int;
                s << pdfMember_int;
                s >> pdfMember_str;
                s.clear();
        }
        //------------------------------------------------------
	// PDF initialization
	pythia.readString("PDF:useLHAPDF = on");
	pythia.readString("PDF:LHAPDFset = " + pdfSet);
	pythia.readString("PDF:LHAPDFmember = " + pdfMember_str);
	// Allow extrapolation of PDF's beyond x and Q2 boundaries, at own risk.
	// Default behaviour is to freeze PDF's at boundaries.
	pythia.readString("PDF:extrapolateLHAPDF = on");
	// Need to change pT0Ref depending on choice of PDF.
	// One possibility: retune to same <n_charged>.
	//pythia.readString("MultipleInteractions:pT0Ref = 10");
	//------------------------------------------------------


	//----------------------------------------------------------------------------------------------
	// to switch on the process ffbar--->gamma/Z
	pythia.readString("WeakSingleBoson:ffbar2gmZ = on");

	//pythia.readString("WeakZ0:gmZmode = 1"); // only photon propagator (can be usefull for tests)
	//pythia.readString("WeakZ0:gmZmode = 2"); // only Z0 propagator (can be usefull for tests)

	// rule out all gamma/Z decays except for muons
	pythia.readString("22:onMode = off"); // switch off all of the gamma decay modes
	pythia.readString("22:onIfAny = 13"); // switch on the gamma->mu-mu+ decay mode only
	pythia.readString("23:onMode = off"); // switch off all of the Z0 decay modes
	pythia.readString("23:onIfAny = 13"); // switch on the Z0->mu-mu+ decay mode only



	bool isSmeared = false;
        int nIsSmeared = 0;
        cout << "select true(1) or smeared(2)...";
        cin >> nIsSmeared;
        isSmeared = (nIsSmeared==2) ? true : false;
	
		
	// phase space cuts

	string sMmin = "";
	string sMmax = "";
        cout << "choose mHat min phasespace...";
        cin >> sMmin;
	cout << "choose mHat max phasespace...";
        cin >> sMmax;
	
        //pythia.readString("PhaseSpace:mHatMin = 1000");
        pythia.readString("PhaseSpace:mHatMin = "+sMmin);
	pythia.readString("PhaseSpace:mHatMax = "+sMmax);
	//pythia.readString("PhaseSpace:mHatMin = 20");  //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	//pythia.readString("PhaseSpace:mHatMax = 200"); //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



	// Optionally only compare cross sections.
	string partonLevelSwitch = "n";
	string ISRswitch = "n";
	string FSRswitch = "n";
	cout << "switch off PartonLevel:all ? (y/n)......";
	cin >> partonLevelSwitch;
	if (partonLevelSwitch == "y") {
		pythia.readString("PartonLevel:all = off");
	}
	else if (partonLevelSwitch == "n") {
		cout << "switch off PartonLevel:ISR ? (y/n)......";
		cin >> ISRswitch;
		if (ISRswitch == "y") {
			pythia.readString("PartonLevel:ISR = off");
		}
		else if (ISRswitch == "n") {
			pythia.readString("PartonLevel:ISR = on");
		}

		cout << "switch off PartonLevel:FSR ? (y/n)......";
		cin >> FSRswitch;
		if (FSRswitch == "y") {
			pythia.readString("PartonLevel:FSR = off");
		}
	}


	// Initialization for LHC with time-based seed.
	pythia.readString("Random:setSeed = on");
	pythia.readString("Random:seed = 0");
	double LHCCME = 14000.;
        cout << "choose LHC center of mass energy in GeV...";
        cin >> LHCCME;
        //pythia.init(2212, 2212, 14000.);
        pythia.init(2212, 2212, LHCCME);
	cout << "Initialization for LHC with time-based seed" << endl;

	// List changes.
	pythia.settings.listChanged();
	pythia.particleData.listChanged();
	//----------------------------------------------------------------------------------------------

	// file
	commonOperations commonOps;
	string dirName = "/";
	//string dirName = "/test/"; // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	parameters prm; // create an instance of the parameters class
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct

	physicalParameters physPrm;
        prm.setParameters(physPrm);


	//extractFromEventRecord extractions;
	double average=0.;
        double sigma=1.;
        double resolution=0.12;   // 12%
        double pTTrackScale=1.e3; // 1 TeV
        bool   printChanges = false;
 	extractFromEventRecord extractions;      
	if(isSmeared) {
                extractions.setSmear(average, sigma, resolution, pTTrackScale, printChanges);
        }
	//runPrm.invMassCut = 20.; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	extractions.setCuts(runPrm);
	bool fullRecord = true;

	string filePath_eventrecord = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"eventRecord_qqbar2sgmZ02mumubar.dat");
	ofstream eventRecord;
	eventRecord.open(filePath_eventrecord.c_str());

	//string dataFileName = "SM_extractedParticles_2trees_1Msample.root";
	

	string str;
	stringstream sstrm;
	string dataFileName;
	int nEvents = 0;
	int sampleSizeSelector = 1;
	cout << "select sample size: 1M(1), L=500fb^-1(2), L=100fb^-1(3), manual(4) OR L=10fb^-1(5)..." << endl;
	cin >> sampleSizeSelector;
	if (sampleSizeSelector < 1  || sampleSizeSelector > 5) {
		cout << "invalid option, set to 4 (default)" << endl;
		sampleSizeSelector = 4;
	}
	else {
		switch(sampleSizeSelector) {
			case 1:
				nEvents = 1000000;
				cout << "sample size is " << nEvents << " (L>> reference sample)" << endl;
				dataFileName = "SM_true_extractedParticles_2trees_1Msample.root";
				if(isSmeared) dataFileName = "SM_smeared_extractedParticles_2trees_1Msample.root";
				break;
			case 2:
				nEvents = 2400;
				cout << "for L=500[fb^-1], sample size is " << nEvents << endl;
				dataFileName = "SM_true_extractedParticles_2trees_500inversefemtobarnSample.root";
				if(isSmeared) dataFileName = "SM_smeared_extractedParticles_2trees_500inversefemtobarnSample.root";
				break;
			case 3:
				//nEvents = 480; // 1->6 TeV with cuts
				//nEvents = 553; // 1->6 TeV without cuts
				nEvents = 864;   // 0.9->14 TeV without cuts
				cout << "for L=100[fb^-1], sample size is " << nEvents << endl;
				dataFileName = "SM_true_extractedParticles_2trees_100inversefemtobarnSample.root";
				if(isSmeared) dataFileName = "SM_smeared_extractedParticles_2trees_100inversefemtobarnSample.root";
				break;
			case 4:
				cout << "\n\nenter the number of events to loop over (integer>0): ";
				cin >> nEvents;
				sstrm.clear();
				sstrm << nEvents;
				sstrm >> str;
				cout << "manual sample size is " << str << endl;
				dataFileName = "SM_true_extractedParticles_2trees_" + str + "_EventsSample.root";
				if(isSmeared) dataFileName = "SM_smeared_extractedParticles_2trees_" + str + "_EventsSample.root";
				sstrm.clear();
				break;
			case 5:
				if(physPrm.mStar==4000.) {
                                        nEvents = 8392;
                                        cout << "for E=7[TeV], 200[GeV]<=mHat<=6[TeV] L=10[fb^-1], sample size is " << nEvents << endl;
                                        dataFileName = "SM_true_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root";
                                        if(isSmeared) dataFileName = "SM_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar4TeV.root";
                                        break;
                                }
                                else if(physPrm.mStar==3000.) {
                                        nEvents = 8392;
                                        cout << "for E=7[TeV], 200[GeV]<=mHat<=6[TeV] L=10[fb^-1], sample size is " << nEvents << endl;
                                        dataFileName = "SM_true_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar3TeV.root";
                                        if(isSmeared) dataFileName = "SM_smeared_extractedParticles_2trees_10inversefemtobarnSample7TeVmStar3TeV.root";
                                        break;
                                }
			default:
				exit(-1);
		}
	}	
	
	//extractions.setOutputFile("MOSESDATA", dirName+"SM_extractedParticles.dat", fullRecord);
	extractions.setOutputFile("MOSESDATA", dirName+dataFileName, fullRecord, "testTree");

	bool write2Tree = true;
        bool writeWithAllPairs = true;


	// run
	cout << "\n\n" << "---------------------------Start the run--------------------------" << endl;


	int mod = (int)nEvents/10;
	int evt=1;
	bool passCuts = true;
	while (evt <= nEvents) {
		pythia.next();
		//--------------------- for the full generation -----------------------------------------------
		//passCuts = extractions.registerEvent(pythia.event,13); // register the muons (absId=13) to file
//		passCuts = extractions.registerEvent(pythia.event,13, write2Tree); // register the muons (absId=13) to tree
		passCuts = extractions.registerEvent(pythia.event,13, write2Tree, writeWithAllPairs); // register the muons (absId=13) to tree and all the pairs that didn't pass the cuts
		//---------------------------------------------------------------------------------------------
		if (evt%mod==0) { cout << "event: " << evt << endl; }
		if (evt==nEvents) {
			pythia.process.list();
			pythia.event.list(eventRecord);
		}
		if (passCuts) { evt++; }
	}
	eventRecord.close();
	extractions.finalizeOutput(true);
	pythia.statistics();
	extractions.printInfo();
}

