#ifndef ANALYSEEVENT_h
#define ANALYSEEVENT_h
/**********************************/
/* a class for general operations */
/**********************************/
//----------------
#include "Moses.h"
//----------------
#include "kinematics.h"
#include "parameters.h"
#include "smearing.h"
//-----------------

using namespace std;
using namespace Pythia8;

namespace Moses1 {

class extractFromEventRecord : public Particle {
	public:
		extractFromEventRecord();
		~extractFromEventRecord();
		
		void printInfo();
	
		void setSmear(double average, double sigma, double resolution, double pTTrackScale, bool printChanges);
		void setCuts(runParameters &runPrm);

		void setOutputFile(string envPath, string fileName, bool fullRecord);
		void setOutputFile(string envPath, string fileName, bool fullRecord, string treeName);

		void extract2vector(Particle &particle);
		void extract2vectors(Particle &particle);
		void extract2vectors(Particle &particle, bool doSmear);

		void registerParticle(Particle &particle);
		void registerParticle(Particle &particle, bool write2tree);
		void registerParticle(Particle &particle, bool writeXbjorken,  bool write2tree);
		void registerParticle(Particle &particle, bool writeXbjorken, bool write2tree, bool passCuts);
		void registerParticle(Particle &particle, Particle &trueParticle, bool writeXbjorken, bool write2tree, bool passCuts);

		void calculateXbjorken(Event &event);

		bool registerEvent(Event &event, int absId);
		bool registerEvent(Event &event, int absId, bool write2tree);
		bool registerEvent(Event &event, int absId, bool write2tree, bool writeWithAllPairs);
		
		void finalizeOutput();
		void finalizeOutput(bool finalizeTree);

	private:
		int info_trueEventsWithOnePair, info_smearedEventsWithOnePair, info_trueEventsPassingCuts,
		    info_smearedEventsPassingCuts, info_eventsWithFlippedCharge, info_eventsWithMoreThanOnePair,
		    info_smearedEventsWithOnePairFailingCuts, info_trueEventsWithOnePairFailingCuts;
	
		bool m_doSmearing;
                smearing m_smearing;
                bool m_printChanges;
	
		commonOperations m_commonOps;
		string m_filePath;
		ofstream m_outFile;
	
		TFile *m_treeFile;
		TTree *m_fullRecordTree;
		TTree *m_reducedRecordTree;
		TTree *m_fullRecordAllPairsTree;
                TTree *m_reducedRecordAllPairsTree;
		TTree *m_fullRecordAllTruePairsTree;
		TTree *m_reducedRecordAllTruePairsTree;

		Particle m_particle;
		Particle m_trueParticle;
		
		int m_eventNumber;

		int m_id;
		double m_m;
		double m_e;
		double m_px;
		double m_py;
		double m_pz;
		double m_pAbs;
		double m_pT;
		double m_pPlus;
		double m_pMinus;
		double m_theta;
		double m_phi;
		double m_y;
		double m_eta;

		double m_xq;
		double m_xqbar;


		int m_tru_id;
                double m_tru_m;
                double m_tru_e;
                double m_tru_px;
                double m_tru_py;
                double m_tru_pz;
                double m_tru_pAbs;
                double m_tru_pT;
                double m_tru_pPlus;
                double m_tru_pMinus;
                double m_tru_theta;
                double m_tru_phi;
                double m_tru_y;
                double m_tru_eta;

                double m_tru_xq;
                double m_tru_xqbar;

		bool m_fullRecord;

		double m_xBjorken_q, m_xBjorken_qbar;

		vector<Particle> m_particleVec, m_Vec, m_antiVec, m_VecTmp, m_antiVecTmp;
		int m_Nparticles, m_Nantiparticles;

		double m_invMassCut, m_pTransverseCut, m_EtaCut, m_pTsystemCut, m_ySystemCut;
		kinematics m_kin;
};

}
#endif

