#ifndef DILEPTONBASICANALYZER_h
#define DILEPTONBASICANALYZER_h
/*********************************************/
//----------------
#include "Moses.h"
//----------------
#include "parameters.h"
#include "smearing.h"

using namespace std;
using namespace Pythia8;

namespace Moses1 {

struct basicRecord {
	double mHat;
	double cosTheta_lab;
};

struct minimalRecord {
	int id;
	double m;
	double e;
	double px;
	double py;
	double pz;
};

struct fullRecord {
	int eventNumber;
	int id;
	double m;
	double e;
	double px;
	double py;
	double pz;
	double pAbs;
	double pT;
	double pPlus;
	double pMinus;
	double theta;
	double phi;
	double y;
	double eta;

	double xq;
	double xqbar;
};


// A derived class for a completely new, user defined (from scratch) process
class dileptonAnalyzer {
	public:
		// Constructor.
		dileptonAnalyzer();
		~dileptonAnalyzer();
	
		void setSmear(double average, double sigma, double resolution, double pTTrackScale, bool printChanges);
		void setCuts(double invMcut, double pTcut, double etacut);
		void setTrue(bool doTrue);	

		void clearAllLeptons();
		int getAllLeptons(vector<fullRecord> &allgoodLeptons);
		int getAllaLeptons(vector<fullRecord> &allgoodaLeptons);
		int getAllLeptonsTrue(vector<fullRecord> &allgoodLeptonsTrue);
                int getAllaLeptonsTrue(vector<fullRecord> &allgoodaLeptonsTrue);
		
		int  loopOverEvents(string filePath);
		int  loopOverEvents(string filePath, string treeName, int nEvents, int nStart);
		
		void readParticleEntry(ifstream &file, fullRecord &fulRec);
		void readParticleEntry(Int_t l, fullRecord &fulRec);
		void readTrueParticleEntry(Int_t l, fullRecord &fulRec);
		void readParticleEntry(ifstream &file, minimalRecord &minRec);
	
		int allGoodLeptons();	
	
		double invMass(fullRecord &lepton, fullRecord &alepton);
		double pT(fullRecord &lepton, fullRecord &alepton);
		double y(fullRecord &lepton, fullRecord &alepton);
		double cosThetaBoost(fullRecord &lepton, fullRecord &alepton);
		double cosThetaCollinsSoper(fullRecord &lepton, fullRecord &alepton);
		double cosThetaCollinsSoper(fullRecord &lepton, fullRecord &alepton, bool quarkDirection);

		bool invMcut(fullRecord &lepton, fullRecord &alepton);
		bool pTcut(fullRecord &lepton, fullRecord &alepton);
		bool etaCut(fullRecord &lepton, fullRecord &alepton);

	private:
		bool m_doSmearing;
		bool m_doTrue;
		smearing m_smearing;
		bool m_printChanges;
	
		vector<fullRecord> m_Lepton, m_aLepton, m_LeptonTrue, m_aLeptonTrue;
		vector<fullRecord> m_allgoodLeptons, m_allgoodaLeptons, m_allgoodLeptonsTrue, m_allgoodaLeptonsTrue;
		fullRecord m_tmpLepton, m_tmpLeptonTrue;
		int m_Nlept, m_Nalept;
	
		int m_events, m_goodLeptons;

		//--------------------------
		TFile *m_treeFile;
		TTree *m_dsigmadmHatTree;
		TTree *m_tru_dsigmadmHatTree;
	
		int m_eventNumber;
		int m_id;
		double m_m;
		double m_e;
		double m_px;
		double m_py;
		double m_pz;
		double m_pAbs;
		double m_pT;
		double m_pPlus;
		double m_pMinus;
		double m_theta;
		double m_phi;
		double m_y;
		double m_eta;

		double m_xq;
		double m_xqbar;

		int m_tru_eventNumber;
		int m_tru_id;
                double m_tru_m;
                double m_tru_e;
                double m_tru_px;
                double m_tru_py;
                double m_tru_pz;
                double m_tru_pAbs;
                double m_tru_pT;
                double m_tru_pPlus;
                double m_tru_pMinus;
                double m_tru_theta;
                double m_tru_phi;
                double m_tru_y;
                double m_tru_eta;
		//--------------------------

		Vec4 m_p1,m_p2,m_pSum;
		
		double m_invMcut, m_pTcut, m_etacut;

};


}

#endif

