#ifndef RAPIDITY_h
#define RAPIDITY_h
/*************************************************/
/* class to evaluate the rapidity y from x1 & x2 */
/* inherit from LHApdf                           */
/*************************************************/
//----------------
#include "Moses.h"
//----------------
#include "parameters.h"
#include "cutsEmulator.h"
#include "mosesParticle.h"


using namespace std;

namespace Moses1 {

class rapidity : public lhapdf {
	public:
		rapidity();
		virtual ~rapidity();
		
		int nActualSegments;
	
		// to return x1 or x2 (colliding quark or antiquark bjorken x)
		virtual double getBjorkenx(double s, double y, runParameters &runPrm, bool isAnti);

		// to return rapidity y from (x1,x2)
		virtual double getrapidity(double x1, double x2);

		virtual int pdtTOlhaID(int pdtID);
		
		// to get ymin and ymax (integration boundaries) depend on s_hat, s_max
		virtual double getrapidityBoundaries(double s, runParameters &runPrm);
		
		virtual double simpsonIntegral(int nMaxSegments, double s, customParticle &parton, runParameters &runPrm, lhapdf &lha);
		virtual double simpsonIntegral(int nMaxSegments, double s, customParticle &parton, runParameters &runPrm, lhapdf &lha, cutsEmulator &cuts);
		virtual double simpsonIntegral(int nMaxSegments, double s, customParticle &parton, runParameters &runPrm, lhapdf &lha, double cosThetaAbs);
		virtual double simpsonIntegral(int nMaxSegments, double s, customParticle &parton, runParameters &runPrm, lhapdf &lha, cutsEmulator &cuts,
								 double Sq, double Aq, bool isAsymmetric);
		virtual double simpsonIntegralTest(int nMaxSegments, double Q, lhapdf &lha);
	private:
		double absBoundary;
		double Q;
		double dy;
		double h;
		double integrand;
		double I;
		double y; // lower y
		double xpdf_q, xpdf_qbar;
		double xpdf1, xpdf2, xpdf3, xpdf4;
		double xpdf1tmp, xpdf2tmp;
		double x_q, x_qbar;
		int flavor_1; // flavor of 'q'
		int flavor_2; // flavor of 'qbar'
		string flavorname;
		double y_highLimit, y_lowLimit;
};

}
#endif
