/**********************************/
/* a class for general operations */
/**********************************/
#include <iostream>
#include <stdlib.h>
#include <stdio.h>	// for the sprintf call
#include <string>
#include <sstream>	// for the int to string operation (stringstream call)
#include <cstring>	// for the string functions
#include <math.h>
#include <fstream>
#include <vector>
//-----------------
#include "mosesCommon.h"

using namespace std;

namespace Moses1 {

commonOperations::commonOperations(){}
commonOperations::~commonOperations(){}


bool commonOperations::fileexists(string filename)
{
	ifstream ifile(filename.c_str());
	return ifile;
}

void commonOperations::exitIfNotExist(string fullPath)
{
	if (!fileexists(fullPath)) {
                cout << "Error: the file " << fullPath  << endl;
                cout << "       does not exist, try to run:" << endl;
                cout << "       ./helper --sum=exeName" << endl;
                cout << "       to see the list of inputs," << endl;
                cout << "       where exeName is the exited executable." << endl;
                cout << "Exit now" << endl;
                exit(-1);
        }
}


// a method to chaeck is the environment veriable is defined and add to it the file name
// to get the full path to the file + its name.
// it will cause an 'exit' status if the environment variable is not defined.
string commonOperations::checkANDsetFilepath(string envPath, string fileName) {
	char * tmpPath = getenv(envPath.c_str());
	if (!tmpPath) {
		cerr << "environment variable " << envPath  <<  " not defined" << endl;
		cerr << "\tdid you forget to 'source configure_bash or configure_tcsh' ?" << endl;
		exit(-1);
	}

	string fullPath = tmpPath;
	fullPath += fileName;

	//----------------- check if there are ".."-like parts in the fullPath
	int i=0;
	int pos=0;
	bool isDoubledot = false;
	while (i<(int)fullPath.size()-1) { // check if there's a ".." string in fullPath
		if (fullPath.substr(i,2) == "..") {
			pos = i;
			isDoubledot = true;
			//cout << "warning: this is a known problem with file paths, fixing it now..." << endl;
		}
		i++;
	}

	if (isDoubledot) {
		string oldPath = fullPath;
		string fin = fullPath.substr(pos+2,(int)fullPath.size()-(pos+2)); // remember everything from after ".."
		string newpath = fullPath.substr(0,pos-1); // remember everything before "/.."
		int newpos = pos-2;
		string isBackslash = newpath.substr(newpos,1);
		while (isBackslash != "/") { // remove the dir name before "/.."
			newpos--;
			isBackslash = newpath.substr(newpos,1);
		}
		fullPath = newpath.substr(0,newpos) + fin;
		//cout << "\t file path was: " << oldPath << endl;
		//cout << "\t corrected to: " << fullPath << endl;
		cout << "file path: " << fullPath << endl;
	}

	if (!isDoubledot) {cout << "file path: " << fullPath << endl;}
	
	return(fullPath);
}

// a method to chaeck is the environment veriable is defined or not
// will NOT exit if the environment variable is not defined.
bool commonOperations::checkPath(string envPath) {
	bool isDefined = false;
	char * tmpPath = getenv(envPath.c_str());
	if (!tmpPath) {
		cerr << "environment variable " << envPath  <<  " not defined" << endl;
		cerr << "\tdid you forget to 'source configure_bash or configure_tcsh' ?" << endl;
		isDefined = false;
	}
	else {isDefined = true;}
	return(isDefined);
}




void commonOperations::SetOKStyle() {
  TStyle* OKStyle = new  TStyle("OKStyle", "OK Default Style");

  // Colors

  //set the background color to white
  OKStyle->SetFillColor(10);
  OKStyle->SetFrameFillColor(10);
  OKStyle->SetFrameFillStyle(0);
  OKStyle->SetFillStyle(0);
  OKStyle->SetCanvasColor(10);
  OKStyle->SetPadColor(10);
  OKStyle->SetTitleFillColor(0);
  OKStyle->SetStatColor(10);

  // Get rid of drop shadow on legends
  // This doesn't seem to work.  Call SetBorderSize(1) directly on your TLegends
  OKStyle->SetLegendBorderSize(0);

  //don't put a colored frame around the plots
  OKStyle->SetFrameBorderMode(0);
  OKStyle->SetCanvasBorderMode(0);
  OKStyle->SetPadBorderMode(0);

  //use the primary color palette
  OKStyle->SetPalette(1,0);

  //set the default line color for a histogram to be black
  OKStyle->SetHistLineColor(kBlack);

  //set the default line color for a fit function to be red
  OKStyle->SetFuncColor(kRed);

  //make the axis labels black
  OKStyle->SetLabelColor(kBlack,"xyz");

  //set the default title color to be black
  OKStyle->SetTitleColor(kBlack);

  //set the margins
  OKStyle->SetPadBottomMargin(0.15);
  OKStyle->SetPadLeftMargin(0.15);
  OKStyle->SetPadTopMargin(0.075);
  OKStyle->SetPadRightMargin(0.15);

  //set axis label and title text sizes
  OKStyle->SetLabelSize(0.05,"xyz");
  OKStyle->SetTitleSize(0.07,"xyz");
  OKStyle->SetTitleOffset(0.9,"xy");
  OKStyle->SetTitleOffset(0.655555,"z");
  OKStyle->SetStatFontSize(0.05);
  OKStyle->SetTextSize(0.07);
  OKStyle->SetTitleBorderSize(0);

  //set line widths
  OKStyle->SetHistLineWidth(1);
  OKStyle->SetFrameLineWidth(2);
  OKStyle->SetFuncWidth(2);

  // Misc

  //align the titles to be centered
  //OKStyle->SetTextAlign(22);

  //turn off xy grids
  OKStyle->SetPadGridX(0);
  OKStyle->SetPadGridY(0);

  //set the tick mark style
  OKStyle->SetPadTickX(1);
  OKStyle->SetPadTickY(1);

  //don't show the fit parameters in a box
  OKStyle->SetOptFit(0000);

  //set the default stats shown
//  OKStyle->SetOptStat("neimr");
  OKStyle->SetOptStat(0);

  //marker settings
  OKStyle->SetMarkerStyle(8);  //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  OKStyle->SetMarkerSize(0.7); //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  // Fonts
  OKStyle->SetStatFont(12);
  OKStyle->SetLabelFont(12,"xyz");
  OKStyle->SetTitleFont(12,"xyz");
  OKStyle->SetTextFont(12);

  // Set the paper size for output
//  OKStyle->SetPaperSize(TStyle::kUSLetter);
  OKStyle->SetPaperSize(TStyle::kA4);

  //done
  OKStyle->cd();

  cout << "Using OKStyle" << endl;
}


TH1D* commonOperations::Shift(TH1D* h, double x=0.1) {
  /// create a new histigram shifted by some amount

  int     N = h->GetNbinsX();
  double ul = h->GetBinLowEdge(N+1);
  double ll = h->GetBinLowEdge(1);
 
  double delta = x*(ul-ll)/N;

  TH1D* h0 = new TH1D("h","h", N, ll-delta, ul-delta);
  h0->SetDirectory(0);

  for ( int i=1 ; i<=N ; i++ ) { 
    h0->SetBinContent(i,h->GetBinContent(i));
    h0->SetBinError(i,h->GetBinError(i));
  }

  return h0;
}


void commonOperations::NormToBinWidth(TH1D* h) { 
  /// normalise to bin width
  for ( int i=1 ; i<=h->GetNbinsX() ; i++ ) { 
    double delta = h->GetBinLowEdge(i+1) - h->GetBinLowEdge(i);
    h->SetBinContent(i,h->GetBinContent(i)/delta);
    h->SetBinError(i,h->GetBinError(i)/delta);
  }
}

double commonOperations::Integral(TH1D* h) { 
  /// get integral
  double N=0;
  for ( int i=1 ; i<=h->GetNbinsX() ; i++ ) N += h->GetBinContent(i);
  return N;
}

double commonOperations::IntegralOverFlow(TH1D* h) { 
  /// get integral including over/underflow
  double N=0;
  for ( int i=0 ; i<=h->GetNbinsX()+1 ; i++ ) N += h->GetBinContent(i);
  return N;
}



void commonOperations::Scale(TH1D* h, double d) { 
  /// scale including over/underflow
  for ( int i=0 ; i<=h->GetNbinsX()+1 ; i++ ) { 
      h->SetBinError(i,h->GetBinError(i)*d);
      h->SetBinContent(i,h->GetBinContent(i)*d);
  }
}


void commonOperations::Norm(TH1D* h) { 
  double N = Integral(h);

  std::cout << "int " << N << " before" << std::endl;

  if ( N>0 ) Scale(h,1/N);

  N = Integral(h);

  std::cout << "int " << N << " after" << std::endl;

}



void commonOperations::plot(TH1D* h) { 

  //  Norm(h3);
  Norm(h);
  NormToBinWidth(h);

  TH1D* h2 = Shift(h, 0.1);

  h->DrawCopy("e1");
  h2->DrawCopy("samee1");

}



void commonOperations::relDiff(TH1D* hInput, TH1D* hReference, TH1D* hRelDiffPos, TH1D* hRelDiffNeg)
{
	Int_t nBins = hInput->GetNbinsX();
	Double_t reldiff = 0.;
	Double_t nRef = 0.;
	Double_t nInp = 0.;

	for (Int_t n=1 ; n<=nBins ; n++) {
		nRef = hReference->GetBinContent(n);
		nInp = hInput->GetBinContent(n);
		if ((nRef+nInp)!=0) { // for not dividing by zero...
			reldiff = (nRef-nInp) / sqrt(nRef + nInp);
			if      (reldiff < 0) { hRelDiffNeg->SetBinContent(n,reldiff); }
			else if (reldiff > 0) { hRelDiffPos->SetBinContent(n,reldiff); }
			else if (reldiff == 0) { 
				hRelDiffPos->SetBinContent(n,0);
				hRelDiffNeg->SetBinContent(n,0);
			}
		}
		else {
			hRelDiffPos->SetBinContent(n,0);
			hRelDiffNeg->SetBinContent(n,0);
		}
	}
}




double commonOperations::simpsonIntegration(vector<double> &yVector, vector<double> &xVector, double startX, double endX){
	int ySize = yVector.size();
	int xSize = xVector.size();
	if (xSize != ySize)
	{
		cout <<"The Vector's  (x and y) do not have the same size!"<< endl;
		exit(-1);
	}
	int arraysSize = ySize;
	double yArray[xSize];
	double xArray[xSize];
	for (int i = 0 ; i< xSize ; i++){
		yArray[i]=yVector[i];
		xArray[i]=xVector[i];
	}
	return simpsonIntegrationFunction(yArray, xArray, arraysSize, startX, endX);
}

double commonOperations::simpsonIntegration(double *yArray, double *xArray, int arraysSize, double startX, double endX)
{
	return simpsonIntegrationFunction(yArray, xArray, arraysSize, startX, endX);
}


double commonOperations::simpsonIntegrationFunction(double *yArray, double *xArray, int arraysSize, double startX, double endX)
{
	int i;
	int nStart;
	int nEnd;
	double h;
	double simpsonTemp=0;
	double integrand=0;
	double dmHat = xArray[1]-xArray[0];
	//cout << "dmHat = " << dmHat << endl;

	//int yArraySize = sizeof(yArray)/sizeof(double);
	//int xArraySize = sizeof(xArray)/sizeof(double);
	//size_t size_of_xArray = (sizeof xArray);///(sizeof xArray[0]);
	//size_t size_of_yArray = (sizeof yArray);///(sizeof yArray[0]);
	//int xArraySize = (int)size_of_xArray;
	//int yArraySize = (int)size_of_yArray;
	//cout << "yArraySize = " << yArraySize << ", xArraySize = " << xArraySize << endl;
	/*
	if (yArraySize != xArraySize)
	{
		cout <<"The Vector's  (x and y) do not have the same size!"<< endl;
		exit(-1);
	}
	*/
	int yArraySize = arraysSize;

	if (dmHat==0.)
	{
		cout << "In simpson integral dx is 0, The integral canot be calculated" << endl;
		exit(-1);
	}
	if ((startX==endX)&&(endX==0))
	{
		nStart=0;
		nEnd = yArraySize;
	}
	else
	{
		nStart = (int)(startX/dmHat);
		nEnd = (int)(endX/dmHat);
	}
	//////////////////end nStart,nEnd///////////////////////////
	h = dmHat/3.0;
	i = nStart;
	//cout <<" i ="<< i << "h= " << h << "nEnd = " <<nEnd << " dx="<<dx<< "VectorSize="<<VectorSize<<endl;
	    while (i<=nEnd) {
				// calculate the integrand:
                integrand = yArray[i];
			    //cout << " $ " << yVector[i] << " $ " ;
                // simpson:
                if (i==nStart || i==nEnd) {                //end and start pionts
                        simpsonTemp += integrand;
                    }
                else if (i%2==0 && i!=nStart && i!=nEnd) { //evan points
                        simpsonTemp += 2*integrand;
                    }
				else {
                        simpsonTemp += 4*integrand;        //odd points
                    }
                // propagate i:
                i++;				
        }

    	//cout << " $&&& " << simpsonTemp << endl ;
        simpsonTemp *= h; // [nb]
	    // return the array
		return  simpsonTemp;
	
}


double commonOperations::getProbability(TH1D* hReference, TH1D* hInput, double confidenceLevel) // not assuming flat
{
	if(confidenceLevel > 100. || confidenceLevel < 0.)
	{
		cerr << "WARNING: confidence level (" << confidenceLevel << ") is invalid, should be between 0. and 100." << endl;
		cerr << "Exitting now" << endl;
		exit(-1);
	}
	confidenceLevel /= 100.;
	
	Int_t bin=1;
	double xLocation = 0.;
	double refProb = 0.;
	double refIntegral = (double)hReference->GetEntries();
	while( bin <= hReference->GetNbinsX() )
	{
		refProb += hReference->GetBinContent(bin) / refIntegral;
		if(refProb > (1.-confidenceLevel))
		{
			refProb -= hReference->GetBinContent(bin) / refIntegral;
			bin--;
			xLocation = hReference->GetBinLowEdge(bin+1);
			break;
		}
		bin++;
	}
	//cout << "in getProbability(TH1D*, TH1D*, double), bin=" << bin << ", value=" << xLocation << ", refProb=" << refProb << endl;
	return probability(hInput, xLocation);
}

double commonOperations::getProbability(TH1D* hInput, double confidenceLevel) // assuming flat
{
	if(confidenceLevel > 100. || confidenceLevel < 0.)
	{
		cerr << "WARNING: confidence level (" << confidenceLevel << ") is invalid, should be between 0. and 100." << endl;
		cerr << "Exitting now" << endl;
		exit(-1);
	}
	confidenceLevel /= 100.;
	double xLocation = 1. - confidenceLevel;
	return probability(hInput, xLocation);
}

double commonOperations::probability(TH1D* hInput, double xLocation)
{
	if(xLocation>1. || xLocation<0.)
	{
		cerr << "WARNING: x location (" << xLocation << ") is invalid, should be between 0. and 1." << endl;
		cerr << "Exitting now" << endl;
		exit(-1);
	}

	Int_t bin=1;
	Double_t nextBinLowEdge = 0.;
	double prob = 0.;
	double inputIntegral = hInput->GetEntries();
	while( bin <= hInput->GetNbinsX() )
	{
		nextBinLowEdge = hInput->GetBinLowEdge(bin+1);
		if(nextBinLowEdge > xLocation)
		{
			bin--;
			nextBinLowEdge = hInput->GetBinLowEdge(bin+1);
			break;
		}
		prob += ( hInput->GetBinContent(bin)/inputIntegral );
		bin++;
	}
	//cout << "in probability(TH1D*, double), bin=" << bin << ", value=" << nextBinLowEdge << ", prob=" << prob << endl;
	return prob;
}


}
